@@ -1,18 +1,15 @@
-# The GDAL C library, OGR exception, and the Field object
 from django.contrib.gis.gdal.base import GDALBase
-from django.contrib.gis.gdal.error import OGRException, OGRIndexError
+from django.contrib.gis.gdal.error import GDALException, OGRIndexError
 from django.contrib.gis.gdal.field import Field
 from django.contrib.gis.gdal.geometries import OGRGeometry, OGRGeomType
-
-# ctypes function prototypes
 from django.contrib.gis.gdal.prototypes import ds as capi, geom as geom_api
-
-from django.utils.encoding import force_bytes, force_text
 from django.utils import six
-from django.utils.six.moves import xrange
+from django.utils.encoding import force_bytes, force_text
+from django.utils.six.moves import range
+
 
 # For more information, see the OGR C API source code:
-#  http://www.gdal.org/ogr/ogr__api_8h.html
+#  http://www.gdal.org/ogr__api_8h.html
 #
 # The OGR_F_* routines are relevant here.
 class Feature(GDALBase):
@@ -20,21 +17,17 @@ class Feature(GDALBase):
     This class that wraps an OGR Feature, needs to be instantiated
     from a Layer object.
     """
+    destructor = capi.destroy_feature
 
-    #### Python 'magic' routines ####
     def __init__(self, feat, layer):
         """
         Initializes Feature from a pointer and its Layer object.
         """
         if not feat:
-            raise OGRException('Cannot create OGR Feature, invalid pointer given.')
+            raise GDALException('Cannot create OGR Feature, invalid pointer given.')
         self.ptr = feat
         self._layer = layer
 
-    def __del__(self):
-        "Releases a reference to this object."
-        if self._ptr: capi.destroy_feature(self._ptr)
-
     def __getitem__(self, index):
         """
         Gets the Field object at the specified index, which may be either
@@ -52,7 +45,7 @@ class Feature(GDALBase):
 
     def __iter__(self):
         "Iterates over each field in the Feature."
-        for i in xrange(self.num_fields):
+        for i in range(self.num_fields):
             yield self[i]
 
     def __len__(self):
@@ -67,7 +60,7 @@ class Feature(GDALBase):
         "Does equivalence testing on the features."
         return bool(capi.feature_equal(self.ptr, other._ptr))
 
-    #### Feature Properties ####
+    # #### Feature Properties ####
     @property
     def encoding(self):
         return self._layer._ds.encoding
@@ -92,7 +85,7 @@ class Feature(GDALBase):
     def fields(self):
         "Returns a list of fields in the Feature."
         return [capi.get_field_name(capi.get_field_defn(self._layer._ldefn, i))
-                for i in xrange(self.num_fields)]
+                for i in range(self.num_fields)]
 
     @property
     def geom(self):
@@ -106,7 +99,7 @@ class Feature(GDALBase):
         "Returns the OGR Geometry Type for this Feture."
         return OGRGeomType(capi.get_fd_geom_type(self._layer._ldefn))
 
-    #### Feature Methods ####
+    # #### Feature Methods ####
     def get(self, field):
         """
         Returns the value of the field, instead of an instance of the Field
