@@ -1,8 +1,8 @@
 from __future__ import unicode_literals
 
 import codecs
+import glob
 import os
-from optparse import make_option
 
 from django.core.management.base import BaseCommand, CommandError
 from django.core.management.utils import find_command, popen_wrapper
@@ -12,9 +12,7 @@ from django.utils._os import npath, upath
 def has_bom(fn):
     with open(fn, 'rb') as f:
         sample = f.read(4)
-    return sample[:3] == b'\xef\xbb\xbf' or \
-        sample.startswith(codecs.BOM_UTF16_LE) or \
-        sample.startswith(codecs.BOM_UTF16_BE)
+    return sample.startswith((codecs.BOM_UTF8, codecs.BOM_UTF16_LE, codecs.BOM_UTF16_BE))
 
 
 def is_writable(path):
@@ -29,10 +27,6 @@ def is_writable(path):
 
 
 class Command(BaseCommand):
-    option_list = BaseCommand.option_list + (
-        make_option('--locale', '-l', dest='locale', action='append',
-                    help='locale(s) to process (e.g. de_AT). Default is to process all. Can be used multiple times.'),
-    )
     help = 'Compiles .po files to .mo files for use with builtin gettext support.'
 
     requires_system_checks = False
@@ -41,9 +35,27 @@ class Command(BaseCommand):
     program = 'msgfmt'
     program_options = ['--check-format']
 
+    def add_arguments(self, parser):
+        parser.add_argument(
+            '--locale', '-l', dest='locale', action='append', default=[],
+            help='Locale(s) to process (e.g. de_AT). Default is to process all. '
+                 'Can be used multiple times.',
+        )
+        parser.add_argument(
+            '--exclude', '-x', dest='exclude', action='append', default=[],
+            help='Locales to exclude. Default is none. Can be used multiple times.',
+        )
+        parser.add_argument(
+            '--use-fuzzy', '-f', dest='fuzzy', action='store_true', default=False,
+            help='Use fuzzy translations.',
+        )
+
     def handle(self, **options):
-        locale = options.get('locale')
-        self.verbosity = int(options.get('verbosity'))
+        locale = options['locale']
+        exclude = options['exclude']
+        self.verbosity = options['verbosity']
+        if options['fuzzy']:
+            self.program_options = self.program_options + ['-f']
 
         if find_command(self.program) is None:
             raise CommandError("Can't find %s. Make sure you have GNU gettext "
@@ -52,7 +64,13 @@ class Command(BaseCommand):
         basedirs = [os.path.join('conf', 'locale'), 'locale']
         if os.environ.get('DJANGO_SETTINGS_MODULE'):
             from django.conf import settings
-            basedirs.extend([upath(path) for path in settings.LOCALE_PATHS])
+            basedirs.extend(upath(path) for path in settings.LOCALE_PATHS)
+
+        # Walk entire tree, looking for locale directories
+        for dirpath, dirnames, filenames in os.walk('.', topdown=True):
+            for dirname in dirnames:
+                if dirname == 'locale':
+                    basedirs.append(os.path.join(dirpath, dirname))
 
         # Gather existing directories.
         basedirs = set(map(os.path.abspath, filter(os.path.isdir, basedirs)))
@@ -62,9 +80,19 @@ class Command(BaseCommand):
                                "checkout or your project or app tree, or with "
                                "the settings module specified.")
 
+        # Build locale list
+        all_locales = []
         for basedir in basedirs:
-            if locale:
-                dirs = [os.path.join(basedir, l, 'LC_MESSAGES') for l in locale]
+            locale_dirs = filter(os.path.isdir, glob.glob('%s/*' % basedir))
+            all_locales.extend(map(os.path.basename, locale_dirs))
+
+        # Account for excluded locales
+        locales = locale or all_locales
+        locales = set(locales) - set(exclude)
+
+        for basedir in basedirs:
+            if locales:
+                dirs = [os.path.join(basedir, l, 'LC_MESSAGES') for l in locales]
             else:
                 dirs = [basedir]
             locations = []
@@ -90,12 +118,13 @@ class Command(BaseCommand):
 
             # Check writability on first location
             if i == 0 and not is_writable(npath(base_path + '.mo')):
-                self.stderr.write("The po files under %s are in a seemingly not "
-                                  "writable location. mo files will not be updated/created." % dirpath)
+                self.stderr.write("The po files under %s are in a seemingly not writable location. "
+                                  "mo files will not be updated/created." % dirpath)
                 return
 
-            args = [self.program] + self.program_options + ['-o',
-                    npath(base_path + '.mo'), npath(base_path + '.po')]
+            args = [self.program] + self.program_options + [
+                '-o', npath(base_path + '.mo'), npath(base_path + '.po')
+            ]
             output, errors, status = popen_wrapper(args)
             if status:
                 if errors:
