@@ -1,7 +1,8 @@
 from django.http import HttpResponse
-from django.template import loader, Context, RequestContext
 from django.utils import six
 
+from .loader import get_template, select_template
+
 
 class ContentNotRenderedError(Exception):
     pass
@@ -10,19 +11,30 @@ class ContentNotRenderedError(Exception):
 class SimpleTemplateResponse(HttpResponse):
     rendering_attrs = ['template_name', 'context_data', '_post_render_callbacks']
 
-    def __init__(self, template, context=None, content_type=None, status=None):
+    def __init__(self, template, context=None, content_type=None, status=None,
+                 charset=None, using=None):
         # It would seem obvious to call these next two members 'template' and
         # 'context', but those names are reserved as part of the test Client
-        # API. To avoid the name collision, we use tricky-to-debug problems
+        # API. To avoid the name collision, we use different names.
         self.template_name = template
         self.context_data = context
 
+        self.using = using
+
         self._post_render_callbacks = []
 
+        # _request stores the current request object in subclasses that know
+        # about requests, like TemplateResponse. It's defined in the base class
+        # to minimize code duplication.
+        # It's called self._request because self.request gets overwritten by
+        # django.test.client.Client. Unlike template_name and context_data,
+        # _request should not be considered part of the public API.
+        self._request = None
+
         # content argument doesn't make sense here because it will be replaced
         # with rendered template so we always pass empty string in order to
         # prevent errors and provide shorter signature.
-        super(SimpleTemplateResponse, self).__init__('', content_type, status)
+        super(SimpleTemplateResponse, self).__init__('', content_type, status, charset=charset)
 
         # _is_rendered tracks whether the template and context has been baked
         # into a final response.
@@ -32,13 +44,11 @@ class SimpleTemplateResponse(HttpResponse):
         self._is_rendered = False
 
     def __getstate__(self):
-        """Pickling support function.
-
-        Ensures that the object can't be pickled before it has been
-        rendered, and that the pickled state only includes rendered
-        data, not the data used to construct the response.
         """
-        obj_dict = super(SimpleTemplateResponse, self).__getstate__()
+        Raise an exception if trying to pickle an unrendered response. Pickle
+        only rendered data, not the data used to construct the response.
+        """
+        obj_dict = self.__dict__.copy()
         if not self._is_rendered:
             raise ContentNotRenderedError('The response content must be '
                                           'rendered before it can be pickled.')
@@ -51,20 +61,14 @@ class SimpleTemplateResponse(HttpResponse):
     def resolve_template(self, template):
         "Accepts a template object, path-to-template or list of paths"
         if isinstance(template, (list, tuple)):
-            return loader.select_template(template)
+            return select_template(template, using=self.using)
         elif isinstance(template, six.string_types):
-            return loader.get_template(template)
+            return get_template(template, using=self.using)
         else:
             return template
 
     def resolve_context(self, context):
-        """Converts context data into a full Context object
-        (assuming it isn't already a Context object).
-        """
-        if isinstance(context, Context):
-            return context
-        else:
-            return Context(context)
+        return context
 
     @property
     def rendered_content(self):
@@ -77,7 +81,7 @@ class SimpleTemplateResponse(HttpResponse):
         """
         template = self.resolve_template(self.template_name)
         context = self.resolve_context(self.context_data)
-        content = template.render(context)
+        content = template.render(context, self._request)
         return content
 
     def add_post_render_callback(self, callback):
@@ -113,15 +117,17 @@ class SimpleTemplateResponse(HttpResponse):
 
     def __iter__(self):
         if not self._is_rendered:
-            raise ContentNotRenderedError('The response content must be '
-                                          'rendered before it can be iterated over.')
+            raise ContentNotRenderedError(
+                'The response content must be rendered before it can be iterated over.'
+            )
         return super(SimpleTemplateResponse, self).__iter__()
 
     @property
     def content(self):
         if not self._is_rendered:
-            raise ContentNotRenderedError('The response content must be '
-                                          'rendered before it can be accessed.')
+            raise ContentNotRenderedError(
+                'The response content must be rendered before it can be accessed.'
+            )
         return super(SimpleTemplateResponse, self).content
 
     @content.setter
@@ -133,24 +139,10 @@ class SimpleTemplateResponse(HttpResponse):
 
 
 class TemplateResponse(SimpleTemplateResponse):
-    rendering_attrs = SimpleTemplateResponse.rendering_attrs + ['_request', '_current_app']
+    rendering_attrs = SimpleTemplateResponse.rendering_attrs + ['_request']
 
     def __init__(self, request, template, context=None, content_type=None,
-            status=None, current_app=None):
-        # self.request gets over-written by django.test.client.Client - and
-        # unlike context_data and template_name the _request should not
-        # be considered part of the public API.
-        self._request = request
-        # As a convenience we'll allow callers to provide current_app without
-        # having to avoid needing to create the RequestContext directly
-        self._current_app = current_app
+                 status=None, charset=None, using=None):
         super(TemplateResponse, self).__init__(
-            template, context, content_type, status)
-
-    def resolve_context(self, context):
-        """Convert context data into a full RequestContext object
-        (assuming it isn't already a Context object).
-        """
-        if isinstance(context, Context):
-            return context
-        return RequestContext(self._request, context, current_app=self._current_app)
+            template, context, content_type, status, charset, using)
+        self._request = request
