@@ -4,8 +4,11 @@ without further escaping in HTML. Marking something as a "safe string" means
 that the producer of the string has already turned characters that should not
 be interpreted by the HTML engine (e.g. '<') into the appropriate entities.
 """
-from django.utils.functional import curry, Promise
+import warnings
+
 from django.utils import six
+from django.utils.deprecation import RemovedInDjango20Warning
+from django.utils.functional import Promise, curry, wraps
 
 
 class EscapeData(object):
@@ -25,6 +28,7 @@ class EscapeText(six.text_type, EscapeData):
     """
     pass
 
+
 if six.PY3:
     EscapeString = EscapeText
 else:
@@ -36,9 +40,9 @@ else:
 class SafeData(object):
     def __html__(self):
         """
-        Returns the html representation of a string.
+        Returns the html representation of a string for interoperability.
 
-        Allows interoperability with other template engines.
+        This allows other template engines to understand Django's SafeData.
         """
         return self
 
@@ -106,6 +110,7 @@ class SafeText(six.text_type, SafeData):
 
     encode = curry(_proxy_method, method=six.text_type.encode)
 
+
 if six.PY3:
     SafeString = SafeText
 else:
@@ -114,19 +119,30 @@ else:
     SafeUnicode = SafeText
 
 
+def _safety_decorator(safety_marker, func):
+    @wraps(func)
+    def wrapped(*args, **kwargs):
+        return safety_marker(func(*args, **kwargs))
+    return wrapped
+
+
 def mark_safe(s):
     """
     Explicitly mark a string as safe for (HTML) output purposes. The returned
     object can be used everywhere a string or unicode object is appropriate.
 
+    If used on a method as a decorator, mark the returned data as safe.
+
     Can be called multiple times on a single string.
     """
-    if isinstance(s, SafeData):
+    if hasattr(s, '__html__'):
         return s
     if isinstance(s, bytes) or (isinstance(s, Promise) and s._delegate_bytes):
         return SafeBytes(s)
     if isinstance(s, (six.text_type, Promise)):
         return SafeText(s)
+    if callable(s):
+        return _safety_decorator(mark_safe, s)
     return SafeString(str(s))
 
 
@@ -138,10 +154,11 @@ def mark_for_escaping(s):
     Can be called multiple times on a single string (the resulting escaping is
     only applied once).
     """
-    if isinstance(s, (SafeData, EscapeData)):
+    warnings.warn('mark_for_escaping() is deprecated.', RemovedInDjango20Warning)
+    if hasattr(s, '__html__') or isinstance(s, EscapeData):
         return s
     if isinstance(s, bytes) or (isinstance(s, Promise) and s._delegate_bytes):
         return EscapeBytes(s)
     if isinstance(s, (six.text_type, Promise)):
         return EscapeText(s)
-    return EscapeBytes(bytes(s))
+    return EscapeString(str(s))
