@@ -17,13 +17,9 @@ from django.core.signals import setting_changed
 from django.dispatch import receiver
 from django.utils import lru_cache, six
 from django.utils._os import upath
-from django.utils.deprecation import RemovedInDjango19Warning
 from django.utils.encoding import force_text
 from django.utils.safestring import SafeData, mark_safe
-from django.utils.six import StringIO
-from django.utils.translation import (
-    LANGUAGE_SESSION_KEY, TranslatorCommentWarning, trim_whitespace,
-)
+from django.utils.translation import LANGUAGE_SESSION_KEY
 
 # Translations are cached in a dictionary for every language.
 # The active translations are stored by threadid to make them thread local.
@@ -40,19 +36,16 @@ CONTEXT_SEPARATOR = "\x04"
 # and RFC 3066, section 2.1
 accept_language_re = re.compile(r'''
         ([A-Za-z]{1,8}(?:-[A-Za-z0-9]{1,8})*|\*)      # "en", "en-au", "x-y-z", "es-419", "*"
-        (?:\s*;\s*q=(0(?:\.\d{,3})?|1(?:.0{,3})?))?   # Optional "q=1.00", "q=0.8"
+        (?:\s*;\s*q=(0(?:\.\d{,3})?|1(?:\.0{,3})?))?  # Optional "q=1.00", "q=0.8"
         (?:\s*,\s*|$)                                 # Multiple accepts per header.
         ''', re.VERBOSE)
 
-language_code_re = re.compile(r'^[a-z]{1,8}(?:-[a-z0-9]{1,8})*$', re.IGNORECASE)
+language_code_re = re.compile(
+    r'^[a-z]{1,8}(?:-[a-z0-9]{1,8})*(?:@[a-z0-9]{1,20})?$',
+    re.IGNORECASE
+)
 
-language_code_prefix_re = re.compile(r'^/([\w-]+)(/|$)')
-
-# some browsers use deprecated locales. refs #18419
-_DJANGO_DEPRECATED_LOCALES = {
-    'zh-cn': 'zh-hans',
-    'zh-tw': 'zh-hant',
-}
+language_code_prefix_re = re.compile(r'^/(\w+([@-]\w+)?)(/|$)')
 
 
 @receiver(setting_changed)
@@ -104,23 +97,42 @@ class DjangoTranslation(gettext_module.GNUTranslations):
     requested language and add a fallback to the default language, if it's
     different from the requested language.
     """
-    def __init__(self, language):
+    domain = 'django'
+
+    def __init__(self, language, domain=None, localedirs=None):
         """Create a GNUTranslations() using many locale directories"""
         gettext_module.GNUTranslations.__init__(self)
+        if domain is not None:
+            self.domain = domain
         self.set_output_charset('utf-8')  # For Python 2 gettext() (#25720)
 
         self.__language = language
         self.__to_language = to_language(language)
         self.__locale = to_locale(language)
         self._catalog = None
+        # If a language doesn't have a catalog, use the Germanic default for
+        # pluralization: anything except one is pluralized.
+        self.plural = lambda n: int(n != 1)
+
+        if self.domain == 'django':
+            if localedirs is not None:
+                # A module-level cache is used for caching 'django' translations
+                warnings.warn("localedirs is ignored when domain is 'django'.", RuntimeWarning)
+                localedirs = None
+            self._init_translation_catalog()
+
+        if localedirs:
+            for localedir in localedirs:
+                translation = self._new_gnu_trans(localedir)
+                self.merge(translation)
+        else:
+            self._add_installed_apps_translations()
 
-        self._init_translation_catalog()
-        self._add_installed_apps_translations()
         self._add_local_translations()
-        if self.__language == settings.LANGUAGE_CODE and self._catalog is None:
+        if self.__language == settings.LANGUAGE_CODE and self.domain == 'django' and self._catalog is None:
             # default lang should have at least one translation file available.
             raise IOError("No translation files found for default language %s." % settings.LANGUAGE_CODE)
-        self._add_fallback()
+        self._add_fallback(localedirs)
         if self._catalog is None:
             # No catalogs found for this language, set an empty catalog.
             self._catalog = {}
@@ -137,7 +149,7 @@ class DjangoTranslation(gettext_module.GNUTranslations):
         references to 'fallback'.
         """
         return gettext_module.translation(
-            domain='django',
+            domain=self.domain,
             localedir=localedir,
             languages=[self.__locale],
             codeset='utf-8',
@@ -161,8 +173,9 @@ class DjangoTranslation(gettext_module.GNUTranslations):
                 "gettext calls at import time.")
         for app_config in app_configs:
             localedir = os.path.join(app_config.path, 'locale')
-            translation = self._new_gnu_trans(localedir)
-            self.merge(translation)
+            if os.path.exists(localedir):
+                translation = self._new_gnu_trans(localedir)
+                self.merge(translation)
 
     def _add_local_translations(self):
         """Merges translations defined in LOCALE_PATHS."""
@@ -170,13 +183,19 @@ class DjangoTranslation(gettext_module.GNUTranslations):
             translation = self._new_gnu_trans(localedir)
             self.merge(translation)
 
-    def _add_fallback(self):
+    def _add_fallback(self, localedirs=None):
         """Sets the GNUTranslations() fallback with the default language."""
         # Don't set a fallback for the default language or any English variant
         # (as it's empty, so it'll ALWAYS fall back to the default language)
         if self.__language == settings.LANGUAGE_CODE or self.__language.startswith('en'):
             return
-        default_translation = translation(settings.LANGUAGE_CODE)
+        if self.domain == 'django':
+            # Get from cache
+            default_translation = translation(settings.LANGUAGE_CODE)
+        else:
+            default_translation = DjangoTranslation(
+                settings.LANGUAGE_CODE, domain=self.domain, localedirs=localedirs
+            )
         self.add_fallback(default_translation)
 
     def merge(self, other):
@@ -202,7 +221,7 @@ class DjangoTranslation(gettext_module.GNUTranslations):
 
 def translation(language):
     """
-    Returns a translation object.
+    Returns a translation object in the default 'django' domain.
     """
     global _translations
     if language not in _translations:
@@ -217,11 +236,6 @@ def activate(language):
     """
     if not language:
         return
-    if language in _DJANGO_DEPRECATED_LOCALES:
-        msg = ("The use of the language code '%s' is deprecated. "
-               "Please use the '%s' translation instead.")
-        warnings.warn(msg % (language, _DJANGO_DEPRECATED_LOCALES[language]),
-                      RemovedInDjango19Warning, stacklevel=2)
     _active.value = translation(language)
 
 
@@ -323,6 +337,7 @@ def gettext(message):
     """
     return do_translate(message, 'gettext')
 
+
 if six.PY3:
     ugettext = gettext
 else:
@@ -370,6 +385,7 @@ def ngettext(singular, plural, number):
     """
     return do_ntranslate(singular, plural, number, 'ngettext')
 
+
 if six.PY3:
     ungettext = ngettext
 else:
@@ -529,211 +545,6 @@ def get_language_from_request(request, check_path=False):
     except LookupError:
         return settings.LANGUAGE_CODE
 
-dot_re = re.compile(r'\S')
-
-
-def blankout(src, char):
-    """
-    Changes every non-whitespace character to the given char.
-    Used in the templatize function.
-    """
-    return dot_re.sub(char, src)
-
-
-context_re = re.compile(r"""^\s+.*context\s+((?:"[^"]*?")|(?:'[^']*?'))\s*""")
-inline_re = re.compile(r"""^\s*trans\s+((?:"[^"]*?")|(?:'[^']*?'))(\s+.*context\s+((?:"[^"]*?")|(?:'[^']*?')))?\s*""")
-block_re = re.compile(r"""^\s*blocktrans(\s+.*context\s+((?:"[^"]*?")|(?:'[^']*?')))?(?:\s+|$)""")
-endblock_re = re.compile(r"""^\s*endblocktrans$""")
-plural_re = re.compile(r"""^\s*plural$""")
-constant_re = re.compile(r"""_\(((?:".*?")|(?:'.*?'))\)""")
-one_percent_re = re.compile(r"""(?<!%)%(?!%)""")
-
-
-def templatize(src, origin=None):
-    """
-    Turns a Django template into something that is understood by xgettext. It
-    does so by translating the Django translation tags into standard gettext
-    function invocations.
-    """
-    from django.template.base import (Lexer, TOKEN_TEXT, TOKEN_VAR,
-        TOKEN_BLOCK, TOKEN_COMMENT, TRANSLATOR_COMMENT_MARK)
-    src = force_text(src, settings.FILE_CHARSET)
-    out = StringIO('')
-    message_context = None
-    intrans = False
-    inplural = False
-    trimmed = False
-    singular = []
-    plural = []
-    incomment = False
-    comment = []
-    lineno_comment_map = {}
-    comment_lineno_cache = None
-
-    def join_tokens(tokens, trim=False):
-        message = ''.join(tokens)
-        if trim:
-            message = trim_whitespace(message)
-        return message
-
-    for t in Lexer(src, origin).tokenize():
-        if incomment:
-            if t.token_type == TOKEN_BLOCK and t.contents == 'endcomment':
-                content = ''.join(comment)
-                translators_comment_start = None
-                for lineno, line in enumerate(content.splitlines(True)):
-                    if line.lstrip().startswith(TRANSLATOR_COMMENT_MARK):
-                        translators_comment_start = lineno
-                for lineno, line in enumerate(content.splitlines(True)):
-                    if translators_comment_start is not None and lineno >= translators_comment_start:
-                        out.write(' # %s' % line)
-                    else:
-                        out.write(' #\n')
-                incomment = False
-                comment = []
-            else:
-                comment.append(t.contents)
-        elif intrans:
-            if t.token_type == TOKEN_BLOCK:
-                endbmatch = endblock_re.match(t.contents)
-                pluralmatch = plural_re.match(t.contents)
-                if endbmatch:
-                    if inplural:
-                        if message_context:
-                            out.write(' npgettext(%r, %r, %r,count) ' % (
-                                message_context,
-                                join_tokens(singular, trimmed),
-                                join_tokens(plural, trimmed)))
-                        else:
-                            out.write(' ngettext(%r, %r, count) ' % (
-                                join_tokens(singular, trimmed),
-                                join_tokens(plural, trimmed)))
-                        for part in singular:
-                            out.write(blankout(part, 'S'))
-                        for part in plural:
-                            out.write(blankout(part, 'P'))
-                    else:
-                        if message_context:
-                            out.write(' pgettext(%r, %r) ' % (
-                                message_context,
-                                join_tokens(singular, trimmed)))
-                        else:
-                            out.write(' gettext(%r) ' % join_tokens(singular,
-                                                                    trimmed))
-                        for part in singular:
-                            out.write(blankout(part, 'S'))
-                    message_context = None
-                    intrans = False
-                    inplural = False
-                    singular = []
-                    plural = []
-                elif pluralmatch:
-                    inplural = True
-                else:
-                    filemsg = ''
-                    if origin:
-                        filemsg = 'file %s, ' % origin
-                    raise SyntaxError(
-                        "Translation blocks must not include other block tags: "
-                        "%s (%sline %d)" % (t.contents, filemsg, t.lineno)
-                    )
-            elif t.token_type == TOKEN_VAR:
-                if inplural:
-                    plural.append('%%(%s)s' % t.contents)
-                else:
-                    singular.append('%%(%s)s' % t.contents)
-            elif t.token_type == TOKEN_TEXT:
-                contents = one_percent_re.sub('%%', t.contents)
-                if inplural:
-                    plural.append(contents)
-                else:
-                    singular.append(contents)
-
-        else:
-            # Handle comment tokens (`{# ... #}`) plus other constructs on
-            # the same line:
-            if comment_lineno_cache is not None:
-                cur_lineno = t.lineno + t.contents.count('\n')
-                if comment_lineno_cache == cur_lineno:
-                    if t.token_type != TOKEN_COMMENT:
-                        for c in lineno_comment_map[comment_lineno_cache]:
-                            filemsg = ''
-                            if origin:
-                                filemsg = 'file %s, ' % origin
-                            warn_msg = ("The translator-targeted comment '%s' "
-                                "(%sline %d) was ignored, because it wasn't the last item "
-                                "on the line.") % (c, filemsg, comment_lineno_cache)
-                            warnings.warn(warn_msg, TranslatorCommentWarning)
-                        lineno_comment_map[comment_lineno_cache] = []
-                else:
-                    out.write('# %s' % ' | '.join(lineno_comment_map[comment_lineno_cache]))
-                comment_lineno_cache = None
-
-            if t.token_type == TOKEN_BLOCK:
-                imatch = inline_re.match(t.contents)
-                bmatch = block_re.match(t.contents)
-                cmatches = constant_re.findall(t.contents)
-                if imatch:
-                    g = imatch.group(1)
-                    if g[0] == '"':
-                        g = g.strip('"')
-                    elif g[0] == "'":
-                        g = g.strip("'")
-                    g = one_percent_re.sub('%%', g)
-                    if imatch.group(2):
-                        # A context is provided
-                        context_match = context_re.match(imatch.group(2))
-                        message_context = context_match.group(1)
-                        if message_context[0] == '"':
-                            message_context = message_context.strip('"')
-                        elif message_context[0] == "'":
-                            message_context = message_context.strip("'")
-                        out.write(' pgettext(%r, %r) ' % (message_context, g))
-                        message_context = None
-                    else:
-                        out.write(' gettext(%r) ' % g)
-                elif bmatch:
-                    for fmatch in constant_re.findall(t.contents):
-                        out.write(' _(%s) ' % fmatch)
-                    if bmatch.group(1):
-                        # A context is provided
-                        context_match = context_re.match(bmatch.group(1))
-                        message_context = context_match.group(1)
-                        if message_context[0] == '"':
-                            message_context = message_context.strip('"')
-                        elif message_context[0] == "'":
-                            message_context = message_context.strip("'")
-                    intrans = True
-                    inplural = False
-                    trimmed = 'trimmed' in t.split_contents()
-                    singular = []
-                    plural = []
-                elif cmatches:
-                    for cmatch in cmatches:
-                        out.write(' _(%s) ' % cmatch)
-                elif t.contents == 'comment':
-                    incomment = True
-                else:
-                    out.write(blankout(t.contents, 'B'))
-            elif t.token_type == TOKEN_VAR:
-                parts = t.contents.split('|')
-                cmatch = constant_re.match(parts[0])
-                if cmatch:
-                    out.write(' _(%s) ' % cmatch.group(1))
-                for p in parts[1:]:
-                    if p.find(':_(') >= 0:
-                        out.write(' %s ' % p.split(':', 1)[1])
-                    else:
-                        out.write(blankout(p, 'F'))
-            elif t.token_type == TOKEN_COMMENT:
-                if t.contents.lstrip().startswith(TRANSLATOR_COMMENT_MARK):
-                    lineno_comment_map.setdefault(t.lineno,
-                                                  []).append(t.contents)
-                    comment_lineno_cache = t.lineno
-            else:
-                out.write(blankout(t.contents, 'X'))
-    return out.getvalue()
-
 
 def parse_accept_lang_header(lang_string):
     """
@@ -751,11 +562,8 @@ def parse_accept_lang_header(lang_string):
         if first:
             return []
         if priority:
-            try:
-                priority = float(priority)
-            except ValueError:
-                return []
-        if not priority:        # if priority is 0.0 at this point make it 1.0
+            priority = float(priority)
+        else:
             priority = 1.0
         result.append((lang, priority))
     result.sort(key=lambda k: k[1], reverse=True)
