@@ -1,4 +1,4 @@
-# coding: utf-8
+# -*- coding: utf-8 -*-
 from django.contrib import admin
 from django.db import models
 
@@ -11,8 +11,9 @@ class Band(models.Model):
     class Meta:
         ordering = ('name',)
 
+
 class Song(models.Model):
-    band = models.ForeignKey(Band)
+    band = models.ForeignKey(Band, models.CASCADE)
     name = models.CharField(max_length=100)
     duration = models.IntegerField()
     other_interpreters = models.ManyToManyField(Band, related_name='covers')
@@ -20,13 +21,16 @@ class Song(models.Model):
     class Meta:
         ordering = ('name',)
 
+
 class SongInlineDefaultOrdering(admin.StackedInline):
     model = Song
 
+
 class SongInlineNewOrdering(admin.StackedInline):
     model = Song
     ordering = ('duration', )
 
+
 class DynOrderingBandAdmin(admin.ModelAdmin):
 
     def get_ordering(self, request):
