@@ -4,12 +4,11 @@ Django permissions model. This allows us to check that the PermissionsMixin
 includes everything that is needed to interact with the ModelBackend.
 """
 from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
-from django.contrib.auth.tests.custom_user import (
-    CustomUserManager, RemoveGroupsAndPermissions,
-)
 from django.db import models
 from django.utils.encoding import python_2_unicode_compatible
 
+from .custom_user import CustomUserManager, RemoveGroupsAndPermissions
+
 
 class CustomPermissionsUserManager(CustomUserManager):
     def create_superuser(self, email, password, date_of_birth):
@@ -30,9 +29,6 @@ with RemoveGroupsAndPermissions():
         USERNAME_FIELD = 'email'
         REQUIRED_FIELDS = ['date_of_birth']
 
-        class Meta:
-            app_label = 'auth'
-
         def get_full_name(self):
             return self.email
 
