@@ -8,6 +8,7 @@ from django.utils.encoding import python_2_unicode_compatible
 class Article(models.Model):
     title = models.CharField(max_length=100)
     pub_date = models.DateTimeField()
+    published_on = models.DateField(null=True)
 
     categories = models.ManyToManyField("Category", related_name="articles")
 
@@ -17,7 +18,7 @@ class Article(models.Model):
 
 @python_2_unicode_compatible
 class Comment(models.Model):
-    article = models.ForeignKey(Article, related_name="comments")
+    article = models.ForeignKey(Article, models.CASCADE, related_name="comments")
     text = models.TextField()
     pub_date = models.DateTimeField()
     approval_date = models.DateTimeField(null=True)
