@@ -2,15 +2,10 @@ from __future__ import unicode_literals
 
 from operator import attrgetter
 
-from django.apps import apps
-from django.apps.registry import Apps
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.sessions.backends.db import SessionStore
 from django.db import models
 from django.db.models import Count
-from django.db.models.query_utils import (
-    DeferredAttribute, deferred_class_factory,
-)
 from django.test import TestCase, override_settings
 
 from .models import (
@@ -98,28 +93,6 @@ class DeferRegressionTest(TestCase):
             list(SimpleItem.objects.annotate(Count('feature')).only('name')),
             list)
 
-    def test_ticket_11936(self):
-        app_config = apps.get_app_config("defer_regress")
-        # Regression for #11936 - get_models should not return deferred models
-        # by default. Run a couple of defer queries so that app registry must
-        # contain some deferred classes. It might contain a lot more classes
-        # depending on the order the tests are ran.
-        list(Item.objects.defer("name"))
-        list(Child.objects.defer("value"))
-        klasses = {model.__name__ for model in app_config.get_models()}
-        self.assertIn("Child", klasses)
-        self.assertIn("Item", klasses)
-        self.assertNotIn("Child_Deferred_value", klasses)
-        self.assertNotIn("Item_Deferred_name", klasses)
-        self.assertFalse(any(k._deferred for k in app_config.get_models()))
-
-        klasses_with_deferred = {model.__name__ for model in app_config.get_models(include_deferred=True)}
-        self.assertIn("Child", klasses_with_deferred)
-        self.assertIn("Item", klasses_with_deferred)
-        self.assertIn("Child_Deferred_value", klasses_with_deferred)
-        self.assertIn("Item_Deferred_name", klasses_with_deferred)
-        self.assertTrue(any(k._deferred for k in app_config.get_models(include_deferred=True)))
-
     @override_settings(SESSION_SERIALIZER='django.contrib.sessions.serializers.PickleSerializer')
     def test_ticket_12163(self):
         # Test for #12163 - Pickling error saving session with unsaved model
@@ -131,7 +104,7 @@ class DeferRegressionTest(TestCase):
         s = SessionStore(SESSION_KEY)
         s.clear()
         s["item"] = item
-        s.save()
+        s.save(must_create=True)
 
         s = SessionStore(SESSION_KEY)
         s.modified = True
@@ -246,54 +219,59 @@ class DeferRegressionTest(TestCase):
         qs = SpecialFeature.objects.only('feature__item__name').select_related('feature__item')
         self.assertEqual(len(qs), 1)
 
-    def test_deferred_class_factory(self):
-        new_class = deferred_class_factory(
-            Item,
-            ('this_is_some_very_long_attribute_name_so_modelname_truncation_is_triggered',))
-        self.assertEqual(
-            new_class.__name__,
-            'Item_Deferred_this_is_some_very_long_attribute_nac34b1f495507dad6b02e2cb235c875e')
-
-    def test_deferred_class_factory_already_deferred(self):
-        deferred_item1 = deferred_class_factory(Item, ('name',))
-        deferred_item2 = deferred_class_factory(deferred_item1, ('value',))
-        self.assertIs(deferred_item2._meta.proxy_for_model, Item)
-        self.assertNotIsInstance(deferred_item2.__dict__.get('name'), DeferredAttribute)
-        self.assertIsInstance(deferred_item2.__dict__.get('value'), DeferredAttribute)
-
-    def test_deferred_class_factory_no_attrs(self):
-        deferred_cls = deferred_class_factory(Item, ())
-        self.assertFalse(deferred_cls._deferred)
-
-    def test_deferred_class_factory_apps_reuse(self):
-        """
-        #25563 - model._meta.apps should be used for caching and
-        retrieval of the created proxy class.
-        """
-        isolated_apps = Apps(['defer_regress'])
-
-        class BaseModel(models.Model):
-            field = models.BooleanField()
-
-            class Meta:
-                apps = isolated_apps
-                app_label = 'defer_regress'
-
-        deferred_model = deferred_class_factory(BaseModel, ['field'])
-        self.assertIs(deferred_model._meta.apps, isolated_apps)
-        self.assertIs(deferred_class_factory(BaseModel, ['field']), deferred_model)
-
 
 class DeferAnnotateSelectRelatedTest(TestCase):
     def test_defer_annotate_select_related(self):
         location = Location.objects.create()
         Request.objects.create(location=location)
-        self.assertIsInstance(list(Request.objects
-            .annotate(Count('items')).select_related('profile', 'location')
-            .only('profile', 'location')), list)
-        self.assertIsInstance(list(Request.objects
-            .annotate(Count('items')).select_related('profile', 'location')
-            .only('profile__profile1', 'location__location1')), list)
-        self.assertIsInstance(list(Request.objects
-            .annotate(Count('items')).select_related('profile', 'location')
-            .defer('request1', 'request2', 'request3', 'request4')), list)
+        self.assertIsInstance(
+            list(Request.objects.annotate(Count('items')).select_related('profile', 'location')
+                 .only('profile', 'location')),
+            list
+        )
+        self.assertIsInstance(
+            list(Request.objects.annotate(Count('items')).select_related('profile', 'location')
+                 .only('profile__profile1', 'location__location1')),
+            list
+        )
+        self.assertIsInstance(
+            list(Request.objects.annotate(Count('items')).select_related('profile', 'location')
+                 .defer('request1', 'request2', 'request3', 'request4')),
+            list
+        )
+
+
+class DeferDeletionSignalsTests(TestCase):
+    senders = [Item, Proxy]
+
+    @classmethod
+    def setUpTestData(cls):
+        cls.item_pk = Item.objects.create(value=1).pk
+
+    def setUp(self):
+        self.pre_delete_senders = []
+        self.post_delete_senders = []
+        for sender in self.senders:
+            models.signals.pre_delete.connect(self.pre_delete_receiver, sender)
+            models.signals.post_delete.connect(self.post_delete_receiver, sender)
+
+    def tearDown(self):
+        for sender in self.senders:
+            models.signals.pre_delete.disconnect(self.pre_delete_receiver, sender)
+            models.signals.post_delete.disconnect(self.post_delete_receiver, sender)
+
+    def pre_delete_receiver(self, sender, **kwargs):
+        self.pre_delete_senders.append(sender)
+
+    def post_delete_receiver(self, sender, **kwargs):
+        self.post_delete_senders.append(sender)
+
+    def test_delete_defered_model(self):
+        Item.objects.only('value').get(pk=self.item_pk).delete()
+        self.assertEqual(self.pre_delete_senders, [Item])
+        self.assertEqual(self.post_delete_senders, [Item])
+
+    def test_delete_defered_proxy_model(self):
+        Proxy.objects.only('value').get(pk=self.item_pk).delete()
+        self.assertEqual(self.pre_delete_senders, [Proxy])
+        self.assertEqual(self.post_delete_senders, [Proxy])
