@@ -143,9 +143,5 @@ class Group(models.Model):
         return '%s' % self.name
 
 
-class Cheese(models.Model):
-    name = models.CharField(max_length=100)
-
-
 class Article(models.Model):
     content = models.TextField()
