@@ -9,7 +9,7 @@ from django.forms import (
     CharField, FileField, Form, ModelChoiceField, ModelForm,
 )
 from django.forms.models import ModelFormMetaclass
-from django.test import TestCase
+from django.test import SimpleTestCase, TestCase
 from django.utils import six
 
 from ..models import (
@@ -80,14 +80,14 @@ class TestModelChoiceField(TestCase):
 
 class TestTicket14567(TestCase):
     """
-    Check that the return values of ModelMultipleChoiceFields are QuerySets
+    The return values of ModelMultipleChoiceFields are QuerySets
     """
     def test_empty_queryset_return(self):
         "If a model's ManyToManyField has blank=True and is saved with no data, a queryset is returned."
         option = ChoiceOptionModel.objects.create(name='default')
         form = OptionalMultiChoiceModelForm({'multi_choice_optional': '', 'multi_choice': [option.pk]})
         self.assertTrue(form.is_valid())
-        # Check that the empty value is a QuerySet
+        # The empty value is a QuerySet
         self.assertIsInstance(form.cleaned_data['multi_choice_optional'], models.query.QuerySet)
         # While we're at it, test whether a QuerySet is returned if there *is* a value.
         self.assertIsInstance(form.cleaned_data['multi_choice'], models.query.QuerySet)
@@ -107,65 +107,76 @@ class ModelFormCallableModelDefault(TestCase):
         ChoiceOptionModel.objects.create(id=1, name='default')
         ChoiceOptionModel.objects.create(id=2, name='option 2')
         ChoiceOptionModel.objects.create(id=3, name='option 3')
-        self.assertHTMLEqual(ChoiceFieldForm().as_p(), """<p><label for="id_choice">Choice:</label> <select name="choice" id="id_choice">
-<option value="1" selected="selected">ChoiceOption 1</option>
+        self.assertHTMLEqual(
+            ChoiceFieldForm().as_p(),
+            """<p><label for="id_choice">Choice:</label> <select name="choice" id="id_choice">
+<option value="1" selected>ChoiceOption 1</option>
 <option value="2">ChoiceOption 2</option>
 <option value="3">ChoiceOption 3</option>
 </select><input type="hidden" name="initial-choice" value="1" id="initial-id_choice" /></p>
 <p><label for="id_choice_int">Choice int:</label> <select name="choice_int" id="id_choice_int">
-<option value="1" selected="selected">ChoiceOption 1</option>
+<option value="1" selected>ChoiceOption 1</option>
 <option value="2">ChoiceOption 2</option>
 <option value="3">ChoiceOption 3</option>
 </select><input type="hidden" name="initial-choice_int" value="1" id="initial-id_choice_int" /></p>
-<p><label for="id_multi_choice">Multi choice:</label> <select multiple="multiple" name="multi_choice" id="id_multi_choice">
-<option value="1" selected="selected">ChoiceOption 1</option>
+<p><label for="id_multi_choice">Multi choice:</label>
+<select multiple="multiple" name="multi_choice" id="id_multi_choice" required>
+<option value="1" selected>ChoiceOption 1</option>
 <option value="2">ChoiceOption 2</option>
 <option value="3">ChoiceOption 3</option>
 </select><input type="hidden" name="initial-multi_choice" value="1" id="initial-id_multi_choice_0" /></p>
-<p><label for="id_multi_choice_int">Multi choice int:</label> <select multiple="multiple" name="multi_choice_int" id="id_multi_choice_int">
-<option value="1" selected="selected">ChoiceOption 1</option>
+<p><label for="id_multi_choice_int">Multi choice int:</label>
+<select multiple="multiple" name="multi_choice_int" id="id_multi_choice_int" required>
+<option value="1" selected>ChoiceOption 1</option>
 <option value="2">ChoiceOption 2</option>
 <option value="3">ChoiceOption 3</option>
-</select><input type="hidden" name="initial-multi_choice_int" value="1" id="initial-id_multi_choice_int_0" /></p>""")
+</select><input type="hidden" name="initial-multi_choice_int" value="1" id="initial-id_multi_choice_int_0" /></p>"""
+        )
 
     def test_initial_instance_value(self):
         "Initial instances for model fields may also be instances (refs #7287)"
         ChoiceOptionModel.objects.create(id=1, name='default')
         obj2 = ChoiceOptionModel.objects.create(id=2, name='option 2')
         obj3 = ChoiceOptionModel.objects.create(id=3, name='option 3')
-        self.assertHTMLEqual(ChoiceFieldForm(initial={
-            'choice': obj2,
-            'choice_int': obj2,
-            'multi_choice': [obj2, obj3],
-            'multi_choice_int': ChoiceOptionModel.objects.exclude(name="default"),
-        }).as_p(), """<p><label for="id_choice">Choice:</label> <select name="choice" id="id_choice">
+        self.assertHTMLEqual(
+            ChoiceFieldForm(initial={
+                'choice': obj2,
+                'choice_int': obj2,
+                'multi_choice': [obj2, obj3],
+                'multi_choice_int': ChoiceOptionModel.objects.exclude(name="default"),
+            }).as_p(),
+            """<p><label for="id_choice">Choice:</label> <select name="choice" id="id_choice">
 <option value="1">ChoiceOption 1</option>
-<option value="2" selected="selected">ChoiceOption 2</option>
+<option value="2" selected>ChoiceOption 2</option>
 <option value="3">ChoiceOption 3</option>
 </select><input type="hidden" name="initial-choice" value="2" id="initial-id_choice" /></p>
 <p><label for="id_choice_int">Choice int:</label> <select name="choice_int" id="id_choice_int">
 <option value="1">ChoiceOption 1</option>
-<option value="2" selected="selected">ChoiceOption 2</option>
+<option value="2" selected>ChoiceOption 2</option>
 <option value="3">ChoiceOption 3</option>
 </select><input type="hidden" name="initial-choice_int" value="2" id="initial-id_choice_int" /></p>
-<p><label for="id_multi_choice">Multi choice:</label> <select multiple="multiple" name="multi_choice" id="id_multi_choice">
+<p><label for="id_multi_choice">Multi choice:</label>
+<select multiple="multiple" name="multi_choice" id="id_multi_choice" required>
 <option value="1">ChoiceOption 1</option>
-<option value="2" selected="selected">ChoiceOption 2</option>
-<option value="3" selected="selected">ChoiceOption 3</option>
+<option value="2" selected>ChoiceOption 2</option>
+<option value="3" selected>ChoiceOption 3</option>
 </select><input type="hidden" name="initial-multi_choice" value="2" id="initial-id_multi_choice_0" />
 <input type="hidden" name="initial-multi_choice" value="3" id="initial-id_multi_choice_1" /></p>
-<p><label for="id_multi_choice_int">Multi choice int:</label> <select multiple="multiple" name="multi_choice_int" id="id_multi_choice_int">
+<p><label for="id_multi_choice_int">Multi choice int:</label>
+<select multiple="multiple" name="multi_choice_int" id="id_multi_choice_int" required>
 <option value="1">ChoiceOption 1</option>
-<option value="2" selected="selected">ChoiceOption 2</option>
-<option value="3" selected="selected">ChoiceOption 3</option>
+<option value="2" selected>ChoiceOption 2</option>
+<option value="3" selected>ChoiceOption 3</option>
 </select><input type="hidden" name="initial-multi_choice_int" value="2" id="initial-id_multi_choice_int_0" />
-<input type="hidden" name="initial-multi_choice_int" value="3" id="initial-id_multi_choice_int_1" /></p>""")
+<input type="hidden" name="initial-multi_choice_int" value="3" id="initial-id_multi_choice_int_1" /></p>"""
+        )
 
 
 class FormsModelTestCase(TestCase):
     def test_unicode_filename(self):
         # FileModel with unicode filename and data #########################
-        f = FileForm(data={}, files={'file1': SimpleUploadedFile('我隻氣墊船裝滿晒鱔.txt', 'मेरी मँडराने वाली नाव सर्पमीनों से भरी ह'.encode('utf-8'))}, auto_id=False)
+        file1 = SimpleUploadedFile('我隻氣墊船裝滿晒鱔.txt', 'मेरी मँडराने वाली नाव सर्पमीनों से भरी ह'.encode('utf-8'))
+        f = FileForm(data={}, files={'file1': file1}, auto_id=False)
         self.assertTrue(f.is_valid())
         self.assertIn('file1', f.cleaned_data)
         m = FileModel.objects.create(file=f.cleaned_data['file1'])
@@ -228,19 +239,20 @@ class FormsModelTestCase(TestCase):
         self.assertEqual(obj.def_date, datetime.date(1999, 3, 2))
 
 
-class RelatedModelFormTests(TestCase):
+class RelatedModelFormTests(SimpleTestCase):
     def test_invalid_loading_order(self):
         """
         Test for issue 10405
         """
         class A(models.Model):
-            ref = models.ForeignKey("B")
+            ref = models.ForeignKey("B", models.CASCADE)
 
         class Meta:
             model = A
             fields = '__all__'
 
-        self.assertRaises(ValueError, ModelFormMetaclass, str('Form'), (ModelForm,), {'Meta': Meta})
+        with self.assertRaises(ValueError):
+            ModelFormMetaclass(str('Form'), (ModelForm,), {'Meta': Meta})
 
         class B(models.Model):
             pass
@@ -250,7 +262,7 @@ class RelatedModelFormTests(TestCase):
         Test for issue 10405
         """
         class C(models.Model):
-            ref = models.ForeignKey("D")
+            ref = models.ForeignKey("D", models.CASCADE)
 
         class D(models.Model):
             pass
@@ -279,7 +291,8 @@ class ManyToManyExclusionTestCase(TestCase):
             'multi_choice_int': [opt1.pk],
         }
         instance = ChoiceFieldModel.objects.create(**initial)
-        instance.multi_choice = instance.multi_choice_int = [opt2, opt3]
+        instance.multi_choice.set([opt2, opt3])
+        instance.multi_choice_int.set([opt2, opt3])
         form = ChoiceFieldExclusionForm(data=data, instance=instance)
         self.assertTrue(form.is_valid())
         self.assertEqual(form.cleaned_data['multi_choice'], data['multi_choice'])
@@ -293,26 +306,31 @@ class ManyToManyExclusionTestCase(TestCase):
 class EmptyLabelTestCase(TestCase):
     def test_empty_field_char(self):
         f = EmptyCharLabelChoiceForm()
-        self.assertHTMLEqual(f.as_p(),
-            """<p><label for="id_name">Name:</label> <input id="id_name" maxlength="10" name="name" type="text" /></p>
+        self.assertHTMLEqual(
+            f.as_p(),
+            """<p><label for="id_name">Name:</label> <input id="id_name" maxlength="10" name="name" type="text" required /></p>
 <p><label for="id_choice">Choice:</label> <select id="id_choice" name="choice">
-<option value="" selected="selected">No Preference</option>
+<option value="" selected>No Preference</option>
 <option value="f">Foo</option>
 <option value="b">Bar</option>
-</select></p>""")
+</select></p>"""
+        )
 
     def test_empty_field_char_none(self):
         f = EmptyCharLabelNoneChoiceForm()
-        self.assertHTMLEqual(f.as_p(),
-            """<p><label for="id_name">Name:</label> <input id="id_name" maxlength="10" name="name" type="text" /></p>
-<p><label for="id_choice_string_w_none">Choice string w none:</label> <select id="id_choice_string_w_none" name="choice_string_w_none">
-<option value="" selected="selected">No Preference</option>
+        self.assertHTMLEqual(
+            f.as_p(),
+            """<p><label for="id_name">Name:</label> <input id="id_name" maxlength="10" name="name" type="text" required /></p>
+<p><label for="id_choice_string_w_none">Choice string w none:</label>
+<select id="id_choice_string_w_none" name="choice_string_w_none">
+<option value="" selected>No Preference</option>
 <option value="f">Foo</option>
 <option value="b">Bar</option>
-</select></p>""")
+</select></p>"""
+        )
 
     def test_save_empty_label_forms(self):
-        # Test that saving a form with a blank choice results in the expected
+        # Saving a form with a blank choice results in the expected
         # value being stored in the database.
         tests = [
             (EmptyCharLabelNoneChoiceForm, 'choice_string_w_none', None),
@@ -330,13 +348,16 @@ class EmptyLabelTestCase(TestCase):
 
     def test_empty_field_integer(self):
         f = EmptyIntegerLabelChoiceForm()
-        self.assertHTMLEqual(f.as_p(),
-            """<p><label for="id_name">Name:</label> <input id="id_name" maxlength="10" name="name" type="text" /></p>
-<p><label for="id_choice_integer">Choice integer:</label> <select id="id_choice_integer" name="choice_integer">
-<option value="" selected="selected">No Preference</option>
+        self.assertHTMLEqual(
+            f.as_p(),
+            """<p><label for="id_name">Name:</label> <input id="id_name" maxlength="10" name="name" type="text" required /></p>
+<p><label for="id_choice_integer">Choice integer:</label>
+<select id="id_choice_integer" name="choice_integer">
+<option value="" selected>No Preference</option>
 <option value="1">Foo</option>
 <option value="2">Bar</option>
-</select></p>""")
+</select></p>"""
+        )
 
     def test_get_display_value_on_none(self):
         m = ChoiceModel.objects.create(name='test', choice='', choice_integer=None)
@@ -346,20 +367,28 @@ class EmptyLabelTestCase(TestCase):
     def test_html_rendering_of_prepopulated_models(self):
         none_model = ChoiceModel(name='none-test', choice_integer=None)
         f = EmptyIntegerLabelChoiceForm(instance=none_model)
-        self.assertHTMLEqual(f.as_p(),
-            """<p><label for="id_name">Name:</label> <input id="id_name" maxlength="10" name="name" type="text" value="none-test"/></p>
-<p><label for="id_choice_integer">Choice integer:</label> <select id="id_choice_integer" name="choice_integer">
-<option value="" selected="selected">No Preference</option>
+        self.assertHTMLEqual(
+            f.as_p(),
+            """<p><label for="id_name">Name:</label>
+<input id="id_name" maxlength="10" name="name" type="text" value="none-test" required /></p>
+<p><label for="id_choice_integer">Choice integer:</label>
+<select id="id_choice_integer" name="choice_integer">
+<option value="" selected>No Preference</option>
 <option value="1">Foo</option>
 <option value="2">Bar</option>
-</select></p>""")
+</select></p>"""
+        )
 
         foo_model = ChoiceModel(name='foo-test', choice_integer=1)
         f = EmptyIntegerLabelChoiceForm(instance=foo_model)
-        self.assertHTMLEqual(f.as_p(),
-            """<p><label for="id_name">Name:</label> <input id="id_name" maxlength="10" name="name" type="text" value="foo-test"/></p>
-<p><label for="id_choice_integer">Choice integer:</label> <select id="id_choice_integer" name="choice_integer">
+        self.assertHTMLEqual(
+            f.as_p(),
+            """<p><label for="id_name">Name:</label>
+<input id="id_name" maxlength="10" name="name" type="text" value="foo-test" required /></p>
+<p><label for="id_choice_integer">Choice integer:</label>
+<select id="id_choice_integer" name="choice_integer">
 <option value="">No Preference</option>
-<option value="1" selected="selected">Foo</option>
+<option value="1" selected>Foo</option>
 <option value="2">Bar</option>
-</select></p>""")
+</select></p>"""
+        )
