@@ -23,7 +23,7 @@ from django.utils.encoding import python_2_unicode_compatible
 class TaggedItem(models.Model):
     """A tag on an item."""
     tag = models.SlugField()
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, models.CASCADE)
     object_id = models.PositiveIntegerField()
 
     content_object = GenericForeignKey()
@@ -42,7 +42,7 @@ class ValuableTaggedItem(TaggedItem):
 class AbstractComparison(models.Model):
     comparative = models.CharField(max_length=50)
 
-    content_type1 = models.ForeignKey(ContentType, related_name="comparative1_set")
+    content_type1 = models.ForeignKey(ContentType, models.CASCADE, related_name="comparative1_set")
     object_id1 = models.PositiveIntegerField()
 
     first_obj = GenericForeignKey(ct_field="content_type1", fk_field="object_id1")
@@ -54,7 +54,7 @@ class Comparison(AbstractComparison):
     A model that tests having multiple GenericForeignKeys. One is defined
     through an inherited abstract model and one defined directly on this class.
     """
-    content_type2 = models.ForeignKey(ContentType, related_name="comparative2_set")
+    content_type2 = models.ForeignKey(ContentType, models.CASCADE, related_name="comparative2_set")
     object_id2 = models.PositiveIntegerField()
 
     other_obj = GenericForeignKey(ct_field="content_type2", fk_field="object_id2")
@@ -88,6 +88,10 @@ class Vegetable(models.Model):
         return self.name
 
 
+class Carrot(Vegetable):
+    pass
+
+
 @python_2_unicode_compatible
 class Mineral(models.Model):
     name = models.CharField(max_length=150)
@@ -114,20 +118,24 @@ class Rock(Mineral):
     tags = GenericRelation(TaggedItem)
 
 
+class ValuableRock(Mineral):
+    tags = GenericRelation(ValuableTaggedItem)
+
+
 class ManualPK(models.Model):
     id = models.IntegerField(primary_key=True)
     tags = GenericRelation(TaggedItem, related_query_name='manualpk')
 
 
 class ForProxyModelModel(models.Model):
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, models.CASCADE)
     object_id = models.PositiveIntegerField()
     obj = GenericForeignKey(for_concrete_model=False)
     title = models.CharField(max_length=255, null=True)
 
 
 class ForConcreteModelModel(models.Model):
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, models.CASCADE)
     object_id = models.PositiveIntegerField()
     obj = GenericForeignKey()
 
@@ -143,6 +151,6 @@ class ProxyRelatedModel(ConcreteRelatedModel):
 
 # To test fix for #7551
 class AllowsNullGFK(models.Model):
-    content_type = models.ForeignKey(ContentType, null=True)
+    content_type = models.ForeignKey(ContentType, models.SET_NULL, null=True)
     object_id = models.PositiveIntegerField(null=True)
     content_object = GenericForeignKey()
