@@ -5,9 +5,10 @@ from django.forms.models import modelform_factory
 from django.test import TestCase, skipIfDBFeature
 
 from .models import (
-    A, Address, B, Board, C, CharLink, Company, Contact, Content, D, Developer,
-    Guild, HasLinkThing, Link, Node, Note, OddRelation1, OddRelation2,
-    Organization, Person, Place, Related, Restaurant, Tag, Team, TextLink,
+    A, Address, B, Board, C, Cafe, CharLink, Company, Contact, Content, D,
+    Developer, Guild, HasLinkThing, Link, Node, Note, OddRelation1,
+    OddRelation2, Organization, Person, Place, Related, Restaurant, Tag, Team,
+    TextLink,
 )
 
 
@@ -48,6 +49,17 @@ class GenericRelationTests(TestCase):
         TextLink.objects.create(content_object=oddrel)
         oddrel.delete()
 
+    def test_coerce_object_id_remote_field_cache_persistence(self):
+        restaurant = Restaurant.objects.create()
+        CharLink.objects.create(content_object=restaurant)
+        charlink = CharLink.objects.latest('pk')
+        self.assertIs(charlink.content_object, charlink.content_object)
+        # If the model (Cafe) uses more than one level of multi-table inheritance.
+        cafe = Cafe.objects.create()
+        CharLink.objects.create(content_object=cafe)
+        charlink = CharLink.objects.latest('pk')
+        self.assertIs(charlink.content_object, charlink.content_object)
+
     def test_q_object_or(self):
         """
         SQL query parameters for generic relations are properly
