@@ -2,12 +2,12 @@ from __future__ import unicode_literals
 
 from django.contrib.auth.decorators import login_required
 from django.core.paginator import Paginator
-from django.core.urlresolvers import reverse, reverse_lazy
+from django.urls import reverse, reverse_lazy
 from django.utils.decorators import method_decorator
 from django.views import generic
 
+from .forms import AuthorForm, ContactForm
 from .models import Artist, Author, Book, BookSigning, Page
-from .test_forms import AuthorForm, ContactForm
 
 
 class CustomTemplateView(generic.TemplateView):
