@@ -1,7 +1,6 @@
+from django.contrib.gis.db import models
 from django.utils.encoding import python_2_unicode_compatible
 
-from ..models import models
-
 
 @python_2_unicode_compatible
 class NamedModel(models.Model):
@@ -11,7 +10,6 @@ class NamedModel(models.Model):
 
     class Meta:
         abstract = True
-        required_db_features = ['gis_enabled']
 
     def __str__(self):
         return self.name
@@ -50,7 +48,6 @@ class SimpleModel(models.Model):
 
     class Meta:
         abstract = True
-        required_db_features = ['gis_enabled']
 
 
 class Point2D(SimpleModel):
