@@ -4,16 +4,13 @@ from xml.dom import minidom
 
 from django.conf import settings
 from django.contrib.sites.models import Site
-from django.test import (
-    TestCase, modify_settings, override_settings, skipUnlessDBFeature,
-)
+from django.test import TestCase, modify_settings, override_settings
 
 from .models import City
 
 
 @modify_settings(INSTALLED_APPS={'append': 'django.contrib.sites'})
 @override_settings(ROOT_URLCONF='gis_tests.geoapp.urls')
-@skipUnlessDBFeature("gis_enabled")
 class GeoFeedTest(TestCase):
     fixtures = ['initial']
 
