@@ -2,22 +2,15 @@ from __future__ import unicode_literals
 
 import datetime
 from decimal import Decimal
-from unittest import skipIf
 
 from django.contrib.humanize.templatetags import humanize
 from django.template import Context, Template, defaultfilters
-from django.test import TestCase, modify_settings, override_settings
+from django.test import SimpleTestCase, modify_settings, override_settings
 from django.utils import translation
 from django.utils.html import escape
 from django.utils.timezone import get_fixed_timezone, utc
 from django.utils.translation import ugettext as _
 
-try:
-    import pytz
-except ImportError:
-    pytz = None
-
-
 # Mock out datetime in some tests so they don't fail occasionally when they
 # run too slow. Use a fixed datetime for datetime.now(). DST change in
 # America/Chicago (the default time zone) happened on March 11th in 2012.
@@ -36,7 +29,7 @@ class MockDateTime(datetime.datetime):
 
 
 @modify_settings(INSTALLED_APPS={'append': 'django.contrib.humanize'})
-class HumanizeTests(TestCase):
+class HumanizeTests(SimpleTestCase):
 
     def humanize_tester(self, test_list, result_list, method, normalize_result_func=escape):
         for test_content, result in zip(test_list, result_list):
@@ -174,7 +167,6 @@ class HumanizeTests(TestCase):
         # As 24h of difference they will never be the same
         self.assertNotEqual(naturalday_one, naturalday_two)
 
-    @skipIf(pytz is None, "this test requires pytz")
     def test_naturalday_uses_localtime(self):
         # Regression for #18504
         # This is 2012-03-08HT19:30:00-06:00 in America/Chicago
