@@ -5,21 +5,20 @@ import os
 
 from django.contrib.contenttypes.models import ContentType
 from django.test import TestCase, override_settings
+from django.utils import six, translation
 from django.utils._os import upath
-from django.utils import six
-from django.utils import translation
 
 
 @override_settings(
     USE_I18N=True,
-    LOCALE_PATHS=(
+    LOCALE_PATHS=[
         os.path.join(os.path.dirname(upath(__file__)), 'locale'),
-    ),
+    ],
     LANGUAGE_CODE='en',
-    LANGUAGES=(
+    LANGUAGES=[
         ('en', 'English'),
         ('fr', 'French'),
-    ),
+    ],
 )
 class ContentTypeTests(TestCase):
     def test_verbose_name(self):
@@ -28,8 +27,3 @@ class ContentTypeTests(TestCase):
             self.assertEqual(six.text_type(company_type), 'Company')
         with translation.override('fr'):
             self.assertEqual(six.text_type(company_type), 'Société')
-
-    def test_field_override(self):
-        company_type = ContentType.objects.get(app_label='i18n', model='company')
-        company_type.name = 'Other'
-        self.assertEqual(six.text_type(company_type), 'Other')
