@@ -13,6 +13,7 @@ from unittest import skipUnless
 
 from django import forms
 from django.conf import settings
+from django.conf.locale import LANG_INFO
 from django.conf.urls.i18n import i18n_patterns
 from django.template import Context, Template
 from django.test import (
@@ -364,6 +365,20 @@ class FormattingTests(SimpleTestCase):
             'l': self.long,
         })
 
+    def test_all_format_strings(self):
+        all_locales = LANG_INFO.keys()
+        some_date = datetime.date(2017, 10, 14)
+        some_datetime = datetime.datetime(2017, 10, 14, 10, 23)
+        for locale in all_locales:
+            with translation.override(locale):
+                self.assertIn('2017', date_format(some_date))  # Uses DATE_FORMAT by default
+                self.assertIn('23', time_format(some_datetime))  # Uses TIME_FORMAT by default
+                self.assertIn('2017', date_format(some_datetime, format=get_format('DATETIME_FORMAT')))
+                self.assertIn('2017', date_format(some_date, format=get_format('YEAR_MONTH_FORMAT')))
+                self.assertIn('14', date_format(some_date, format=get_format('MONTH_DAY_FORMAT')))
+                self.assertIn('2017', date_format(some_date, format=get_format('SHORT_DATE_FORMAT')))
+                self.assertIn('2017', date_format(some_datetime, format=get_format('SHORT_DATETIME_FORMAT')))
+
     def test_locale_independent(self):
         """
         Localization of numbers
