@@ -18,7 +18,7 @@ class User(models.Model):
 class Issue(models.Model):
     num = models.IntegerField()
     cc = models.ManyToManyField(User, blank=True, related_name='test_issue_cc')
-    client = models.ForeignKey(User, related_name='test_issue_client')
+    client = models.ForeignKey(User, models.CASCADE, related_name='test_issue_client')
 
     def __str__(self):
         return six.text_type(self.num)
