@@ -17,8 +17,8 @@ class Migration(migrations.Migration):
             name='A3',
             fields=[
                 ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
-                ('b2', models.ForeignKey(to='lookuperror_b.B2')),
-                ('c2', models.ForeignKey(to='lookuperror_c.C2')),
+                ('b2', models.ForeignKey('lookuperror_b.B2', models.CASCADE)),
+                ('c2', models.ForeignKey('lookuperror_c.C2', models.CASCADE)),
             ],
         ),
     ]
