@@ -24,7 +24,7 @@ class Migration(migrations.Migration):
             "Tribble",
             [
                 ("id", models.AutoField(primary_key=True)),
-                ("author", models.ForeignKey(to=settings.AUTH_USER_MODEL, to_field="id")),
+                ("author", models.ForeignKey(settings.AUTH_USER_MODEL, models.CASCADE, to_field="id")),
             ],
         )
 
