@@ -28,7 +28,7 @@ class Migration(migrations.Migration):
             "Book",
             [
                 ("id", models.AutoField(primary_key=True)),
-                ("author", models.ForeignKey("migrations.Author", null=True)),
+                ("author", models.ForeignKey("migrations.Author", models.SET_NULL, null=True)),
             ],
         ),
 
