@@ -5,9 +5,13 @@ class Book(models.Model):
     title = models.CharField(max_length=50)
     author = models.CharField(max_length=50)
     pages = models.IntegerField(db_column='page_count')
+    isbn = models.CharField(max_length=50, db_tablespace='idx_tbls')
 
     class Meta:
-        indexes = [models.indexes.Index(fields=['title'])]
+        indexes = [
+            models.indexes.Index(fields=['title']),
+            models.indexes.Index(fields=['isbn', 'id']),
+        ]
 
 
 class AbstractModel(models.Model):
