@@ -22,10 +22,10 @@ from .models import (
 class ModelInheritanceTest(TestCase):
     def test_model_inheritance(self):
         # Regression for #7350, #7202
-        # Check that when you create a Parent object with a specific reference
-        # to an existent child instance, saving the Parent doesn't duplicate
-        # the child. This behavior is only activated during a raw save - it
-        # is mostly relevant to deserialization, but any sort of CORBA style
+        # When you create a Parent object with a specific reference to an
+        # existent child instance, saving the Parent doesn't duplicate the
+        # child. This behavior is only activated during a raw save - it is
+        # mostly relevant to deserialization, but any sort of CORBA style
         # 'narrow()' API would require a similar approach.
 
         # Create a child-parent-grandparent chain
@@ -49,7 +49,7 @@ class ModelInheritanceTest(TestCase):
         park = ParkingLot(parent=place2, capacity=100)
         park.save_base(raw=True)
 
-        # Check that no extra parent objects have been created.
+        # No extra parent objects have been created.
         places = list(Place.objects.all())
         self.assertEqual(places, [place1, place2])
 
@@ -325,8 +325,8 @@ class ModelInheritanceTest(TestCase):
             assignee="adrian")
 
     def test_abstract_base_class_m2m_relation_inheritance(self):
-        # Check that many-to-many relations defined on an abstract base class
-        # are correctly inherited (and created) on the child class.
+        # many-to-many relations defined on an abstract base class are
+        # correctly inherited (and created) on the child class.
         p1 = Person.objects.create(name='Alice')
         p2 = Person.objects.create(name='Bob')
         p3 = Person.objects.create(name='Carol')
@@ -348,8 +348,8 @@ class ModelInheritanceTest(TestCase):
         parties = list(p2.bachelorparty_set.all())
         self.assertEqual(parties, [bachelor])
 
-        # Check that a subclass of a subclass of an abstract model doesn't get
-        # its own accessor.
+        # A subclass of a subclass of an abstract model doesn't get its own
+        # accessor.
         self.assertFalse(hasattr(p2, 'messybachelorparty_set'))
 
         # ... but it does inherit the m2m from its parent
@@ -410,8 +410,8 @@ class ModelInheritanceTest(TestCase):
 
     def test_inherited_unique_field_with_form(self):
         """
-        Test that a model which has different primary key for the parent model
-        passes unique field checking correctly. Refs #17615.
+        A model which has different primary key for the parent model passes
+        unique field checking correctly (#17615).
         """
         class ProfileForm(forms.ModelForm):
             class Meta:
@@ -420,8 +420,7 @@ class ModelInheritanceTest(TestCase):
 
         User.objects.create(username="user_only")
         p = Profile.objects.create(username="user_with_profile")
-        form = ProfileForm({'username': "user_with_profile", 'extra': "hello"},
-                           instance=p)
+        form = ProfileForm({'username': "user_with_profile", 'extra': "hello"}, instance=p)
         self.assertTrue(form.is_valid())
 
     def test_inheritance_joins(self):
@@ -441,10 +440,7 @@ class ModelInheritanceTest(TestCase):
         self.assertEqual(str(qs.query).count('JOIN'), 1)
 
     def test_issue_21554(self):
-        senator = Senator.objects.create(
-            name='John Doe', title='X', state='Y'
-        )
-
+        senator = Senator.objects.create(name='John Doe', title='X', state='Y')
         senator = Senator.objects.get(pk=senator.pk)
         self.assertEqual(senator.name, 'John Doe')
         self.assertEqual(senator.title, 'X')
@@ -478,6 +474,17 @@ class ModelInheritanceTest(TestCase):
         jane = Supplier.objects.order_by("name").select_related("restaurant")[0]
         self.assertEqual(jane.restaurant.name, "Craft")
 
+    def test_filter_with_parent_fk(self):
+        r = Restaurant.objects.create()
+        s = Supplier.objects.create(restaurant=r)
+        # The mismatch between Restaurant and Place is intentional (#28175).
+        self.assertSequenceEqual(Supplier.objects.filter(restaurant__in=Place.objects.all()), [s])
+
+    def test_ptr_accessor_assigns_state(self):
+        r = Restaurant.objects.create()
+        self.assertIs(r.place_ptr._state.adding, False)
+        self.assertEqual(r.place_ptr._state.db, 'default')
+
     def test_related_filtering_query_efficiency_ticket_15844(self):
         r = Restaurant.objects.create(
             name="Guido's House of Pasta",
@@ -487,12 +494,38 @@ class ModelInheritanceTest(TestCase):
         )
         s = Supplier.objects.create(restaurant=r)
         with self.assertNumQueries(1):
-            self.assertQuerysetEqual(
-                Supplier.objects.filter(restaurant=r),
-                [s], lambda x: x,
-            )
+            self.assertSequenceEqual(Supplier.objects.filter(restaurant=r), [s])
         with self.assertNumQueries(1):
-            self.assertQuerysetEqual(
-                r.supplier_set.all(),
-                [s], lambda x: x,
-            )
+            self.assertSequenceEqual(r.supplier_set.all(), [s])
+
+    def test_queries_on_parent_access(self):
+        italian_restaurant = ItalianRestaurant.objects.create(
+            name="Guido's House of Pasta",
+            address='944 W. Fullerton',
+            serves_hot_dogs=True,
+            serves_pizza=False,
+            serves_gnocchi=True,
+        )
+
+        # No queries are made when accessing the parent objects.
+        italian_restaurant = ItalianRestaurant.objects.get(pk=italian_restaurant.pk)
+        with self.assertNumQueries(0):
+            restaurant = italian_restaurant.restaurant_ptr
+            self.assertEqual(restaurant.place_ptr.restaurant, restaurant)
+            self.assertEqual(restaurant.italianrestaurant, italian_restaurant)
+
+        # One query is made when accessing the parent objects when the instance
+        # is deferred.
+        italian_restaurant = ItalianRestaurant.objects.only('serves_gnocchi').get(pk=italian_restaurant.pk)
+        with self.assertNumQueries(1):
+            restaurant = italian_restaurant.restaurant_ptr
+            self.assertEqual(restaurant.place_ptr.restaurant, restaurant)
+            self.assertEqual(restaurant.italianrestaurant, italian_restaurant)
+
+        # No queries are made when accessing the parent objects when the
+        # instance has deferred a field not present in the parent table.
+        italian_restaurant = ItalianRestaurant.objects.defer('serves_gnocchi').get(pk=italian_restaurant.pk)
+        with self.assertNumQueries(0):
+            restaurant = italian_restaurant.restaurant_ptr
+            self.assertEqual(restaurant.place_ptr.restaurant, restaurant)
+            self.assertEqual(restaurant.italianrestaurant, italian_restaurant)
