@@ -231,11 +231,10 @@ class OrLookupsTests(TestCase):
             3
         )
 
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Article.objects.filter(Q(headline__startswith='Hello'), Q(headline__contains='bye')).values(), [
                 {"headline": "Hello and goodbye", "id": self.a3, "pub_date": datetime(2005, 11, 29)},
             ],
-            lambda o: o,
         )
 
         self.assertEqual(
