@@ -5,13 +5,14 @@ import uuid
 
 from django import forms
 from django.core import exceptions, serializers, validators
+from django.core.exceptions import FieldError
 from django.core.management import call_command
 from django.db import IntegrityError, connection, models
-from django.test import TransactionTestCase, override_settings
+from django.test import TransactionTestCase, modify_settings, override_settings
 from django.test.utils import isolate_apps
 from django.utils import timezone
 
-from . import PostgreSQLTestCase
+from . import PostgreSQLTestCase, PostgreSQLWidgetTestCase
 from .models import (
     ArrayFieldSubclass, CharArrayModel, DateTimeArrayModel, IntegerArrayModel,
     NestedIntegerArrayModel, NullableIntegerArrayModel, OtherTypesArrayModel,
@@ -175,18 +176,54 @@ class TestQuerying(PostgreSQLTestCase):
             self.objs[:2]
         )
 
+    @unittest.expectedFailure
+    def test_in_including_F_object(self):
+        # This test asserts that Array objects passed to filters can be
+        # constructed to contain F objects. This currently doesn't work as the
+        # psycopg2 mogrify method that generates the ARRAY() syntax is
+        # expecting literals, not column references (#27095).
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.filter(field__in=[[models.F('id')]]),
+            self.objs[:2]
+        )
+
+    def test_in_as_F_object(self):
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.filter(field__in=[models.F('field')]),
+            self.objs[:4]
+        )
+
     def test_contained_by(self):
         self.assertSequenceEqual(
             NullableIntegerArrayModel.objects.filter(field__contained_by=[1, 2]),
             self.objs[:2]
         )
 
+    @unittest.expectedFailure
+    def test_contained_by_including_F_object(self):
+        # This test asserts that Array objects passed to filters can be
+        # constructed to contain F objects. This currently doesn't work as the
+        # psycopg2 mogrify method that generates the ARRAY() syntax is
+        # expecting literals, not column references (#27095).
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.filter(field__contained_by=[models.F('id'), 2]),
+            self.objs[:2]
+        )
+
     def test_contains(self):
         self.assertSequenceEqual(
             NullableIntegerArrayModel.objects.filter(field__contains=[2]),
             self.objs[1:3]
         )
 
+    def test_icontains(self):
+        # Using the __icontains lookup with ArrayField is inefficient.
+        instance = CharArrayModel.objects.create(field=['FoO'])
+        self.assertSequenceEqual(
+            CharArrayModel.objects.filter(field__icontains='foo'),
+            [instance]
+        )
+
     def test_contains_charfield(self):
         # Regression for #22907
         self.assertSequenceEqual(
@@ -279,6 +316,15 @@ class TestQuerying(PostgreSQLTestCase):
             [self.objs[3]]
         )
 
+    def test_unsupported_lookup(self):
+        msg = "Unsupported lookup '0_bar' for ArrayField or join on the field not permitted."
+        with self.assertRaisesMessage(FieldError, msg):
+            list(NullableIntegerArrayModel.objects.filter(field__0_bar=[2]))
+
+        msg = "Unsupported lookup '0bar' for ArrayField or join on the field not permitted."
+        with self.assertRaisesMessage(FieldError, msg):
+            list(NullableIntegerArrayModel.objects.filter(field__0bar=[2]))
+
 
 class TestDateTimeExactQuerying(PostgreSQLTestCase):
 
@@ -403,6 +449,7 @@ class TestMigrations(TransactionTestCase):
         name, path, args, kwargs = field.deconstruct()
         new = ArrayField(*args, **kwargs)
         self.assertEqual(type(new.base_field), type(field.base_field))
+        self.assertIsNot(new.base_field, field.base_field)
 
     def test_deconstruct_with_size(self):
         field = ArrayField(models.IntegerField(), size=3)
@@ -450,16 +497,20 @@ class TestMigrations(TransactionTestCase):
         table_name = 'postgres_tests_chartextarrayindexmodel'
         call_command('migrate', 'postgres_tests', verbosity=0)
         with connection.cursor() as cursor:
-            like_constraint_field_names = [
-                c.rsplit('_', 2)[0][len(table_name) + 1:]
-                for c in connection.introspection.get_constraints(cursor, table_name)
-                if c.endswith('_like')
+            like_constraint_columns_list = [
+                v['columns']
+                for k, v in list(connection.introspection.get_constraints(cursor, table_name).items())
+                if k.endswith('_like')
             ]
         # Only the CharField should have a LIKE index.
-        self.assertEqual(like_constraint_field_names, ['char2'])
-        with connection.cursor() as cursor:
-            indexes = connection.introspection.get_indexes(cursor, table_name)
+        self.assertEqual(like_constraint_columns_list, [['char2']])
         # All fields should have regular indexes.
+        with connection.cursor() as cursor:
+            indexes = [
+                c['columns'][0]
+                for c in connection.introspection.get_constraints(cursor, table_name).values()
+                if c['index'] and len(c['columns']) == 1
+            ]
         self.assertIn('char', indexes)
         self.assertIn('char2', indexes)
         self.assertIn('text', indexes)
@@ -635,6 +686,11 @@ class TestSimpleFormField(PostgreSQLTestCase):
         self.assertIsInstance(form_field, SimpleArrayField)
         self.assertEqual(form_field.max_length, 4)
 
+    def test_already_converted_value(self):
+        field = SimpleArrayField(forms.CharField())
+        vals = ['a', 'b', 'c']
+        self.assertEqual(field.clean(vals), vals)
+
 
 class TestSplitFormField(PostgreSQLTestCase):
 
@@ -693,6 +749,8 @@ class TestSplitFormField(PostgreSQLTestCase):
         with self.assertRaisesMessage(exceptions.ValidationError, msg):
             SplitArrayField(forms.IntegerField(max_value=100), size=2).clean([0, 101])
 
+    # To locate the widget's template.
+    @modify_settings(INSTALLED_APPS={'append': 'django.contrib.postgres'})
     def test_rendering(self):
         class SplitForm(forms.Form):
             array = SplitArrayField(forms.CharField(), size=3)
@@ -731,7 +789,63 @@ class TestSplitFormField(PostgreSQLTestCase):
         self.assertEqual(obj.field, [1, 2])
 
 
-class TestSplitFormWidget(PostgreSQLTestCase):
+class TestSplitFormWidget(PostgreSQLWidgetTestCase):
+
+    def test_get_context(self):
+        self.assertEqual(
+            SplitArrayWidget(forms.TextInput(), size=2).get_context('name', ['val1', 'val2']),
+            {
+                'widget': {
+                    'name': 'name',
+                    'is_hidden': False,
+                    'required': False,
+                    'value': "['val1', 'val2']",
+                    'attrs': {},
+                    'template_name': 'postgres/widgets/split_array.html',
+                    'subwidgets': [
+                        {
+                            'name': 'name_0',
+                            'is_hidden': False,
+                            'required': False,
+                            'value': 'val1',
+                            'attrs': {},
+                            'template_name': 'django/forms/widgets/text.html',
+                            'type': 'text',
+                        },
+                        {
+                            'name': 'name_1',
+                            'is_hidden': False,
+                            'required': False,
+                            'value': 'val2',
+                            'attrs': {},
+                            'template_name': 'django/forms/widgets/text.html',
+                            'type': 'text',
+                        },
+                    ]
+                }
+            }
+        )
+
+    def test_render(self):
+        self.check_html(
+            SplitArrayWidget(forms.TextInput(), size=2), 'array', None,
+            """
+            <input name="array_0" type="text" />
+            <input name="array_1" type="text" />
+            """
+        )
+
+    def test_render_attrs(self):
+        self.check_html(
+            SplitArrayWidget(forms.TextInput(), size=2),
+            'array', ['val1', 'val2'], attrs={'id': 'foo'},
+            html=(
+                """
+                <input id="foo_0" name="array_0" type="text" value="val1" />
+                <input id="foo_1" name="array_1" type="text" value="val2" />
+                """
+            )
+        )
 
     def test_value_omitted_from_data(self):
         widget = SplitArrayWidget(forms.TextInput(), size=2)
