@@ -11,6 +11,7 @@ from django.core.handlers.wsgi import LimitedStream, WSGIRequest
 from django.http import (
     HttpRequest, HttpResponse, RawPostDataException, UnreadablePostError,
 )
+from django.http.request import split_domain_port
 from django.test import RequestFactory, SimpleTestCase, override_settings
 from django.test.client import FakePayload
 from django.test.utils import freeze_time, str_prefix
@@ -95,9 +96,8 @@ class RequestsTests(SimpleTestCase):
 
     def test_wsgirequest_with_script_name(self):
         """
-        Ensure that the request's path is correctly assembled, regardless of
-        whether or not the SCRIPT_NAME has a trailing slash.
-        Refs #20169.
+        The request's path is correctly assembled, regardless of whether or
+        not the SCRIPT_NAME has a trailing slash (#20169).
         """
         # With trailing slash
         request = WSGIRequest({
@@ -120,8 +120,7 @@ class RequestsTests(SimpleTestCase):
         """
         WSGI squashes multiple successive slashes in PATH_INFO, WSGIRequest
         should take that into account when populating request.path and
-        request.META['SCRIPT_NAME'].
-        Refs #17133.
+        request.META['SCRIPT_NAME'] (#17133).
         """
         request = WSGIRequest({
             'SCRIPT_URL': '/mst/milestones//accounts/login//help',
@@ -134,9 +133,8 @@ class RequestsTests(SimpleTestCase):
 
     def test_wsgirequest_with_force_script_name(self):
         """
-        Ensure that the FORCE_SCRIPT_NAME setting takes precedence over the
-        request's SCRIPT_NAME environment parameter.
-        Refs #20169.
+        The FORCE_SCRIPT_NAME setting takes precedence over the request's
+        SCRIPT_NAME environment parameter (#20169).
         """
         with override_settings(FORCE_SCRIPT_NAME='/FORCED_PREFIX/'):
             request = WSGIRequest({
@@ -149,9 +147,8 @@ class RequestsTests(SimpleTestCase):
 
     def test_wsgirequest_path_with_force_script_name_trailing_slash(self):
         """
-        Ensure that the request's path is correctly assembled, regardless of
-        whether or not the FORCE_SCRIPT_NAME setting has a trailing slash.
-        Refs #20169.
+        The request's path is correctly assembled, regardless of whether or not
+        the FORCE_SCRIPT_NAME setting has a trailing slash (#20169).
         """
         # With trailing slash
         with override_settings(FORCE_SCRIPT_NAME='/FORCED_PREFIX/'):
@@ -173,8 +170,8 @@ class RequestsTests(SimpleTestCase):
         self.assertEqual(repr(request), str_prefix("<WSGIRequest: GET '/somepath/'>"))
 
     def test_wsgirequest_path_info(self):
-        def wsgi_str(path_info):
-            path_info = path_info.encode('utf-8')           # Actual URL sent by the browser (bytestring)
+        def wsgi_str(path_info, encoding='utf-8'):
+            path_info = path_info.encode(encoding)           # Actual URL sent by the browser (bytestring)
             if six.PY3:
                 path_info = path_info.decode('iso-8859-1')  # Value in the WSGI environ dict (native string)
             return path_info
@@ -182,6 +179,16 @@ class RequestsTests(SimpleTestCase):
         request = WSGIRequest({'PATH_INFO': wsgi_str("/سلام/"), 'REQUEST_METHOD': 'get', 'wsgi.input': BytesIO(b'')})
         self.assertEqual(request.path, "/سلام/")
 
+        # The URL may be incorrectly encoded in a non-UTF-8 encoding (#26971)
+        request = WSGIRequest({
+            'PATH_INFO': wsgi_str("/café/", encoding='iso-8859-1'),
+            'REQUEST_METHOD': 'get',
+            'wsgi.input': BytesIO(b''),
+        })
+        # Since it's impossible to decide the (wrong) encoding of the URL, it's
+        # left percent-encoded in the path.
+        self.assertEqual(request.path, "/caf%E9/")
+
     def test_httprequest_location(self):
         request = HttpRequest()
         self.assertEqual(
@@ -240,7 +247,7 @@ class RequestsTests(SimpleTestCase):
         datetime_cookie = response.cookies['datetime']
         self.assertIn(
             datetime_cookie['expires'],
-            # Slight time dependency; refs #23450
+            # assertIn accounts for slight time dependency (#23450)
             ('Sat, 01-Jan-2028 04:05:06 GMT', 'Sat, 01-Jan-2028 04:05:07 GMT')
         )
 
@@ -526,6 +533,25 @@ class RequestsTests(SimpleTestCase):
         self.assertEqual(request.read(13), b'--boundary\r\nC')
         self.assertEqual(request.POST, {'name': ['value']})
 
+    def test_POST_immutable_for_mutipart(self):
+        """
+        MultiPartParser.parse() leaves request.POST immutable.
+        """
+        payload = FakePayload("\r\n".join([
+            '--boundary',
+            'Content-Disposition: form-data; name="name"',
+            '',
+            'value',
+            '--boundary--',
+        ]))
+        request = WSGIRequest({
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'multipart/form-data; boundary=boundary',
+            'CONTENT_LENGTH': len(payload),
+            'wsgi.input': payload,
+        })
+        self.assertFalse(request.POST._mutable)
+
     def test_POST_connection_error(self):
         """
         If wsgi.input.read() raises an exception while trying to read() the
@@ -544,6 +570,28 @@ class RequestsTests(SimpleTestCase):
         with self.assertRaises(UnreadablePostError):
             request.body
 
+    def test_set_encoding_clears_POST(self):
+        payload = FakePayload('name=Hello Günter')
+        request = WSGIRequest({
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'application/x-www-form-urlencoded',
+            'CONTENT_LENGTH': len(payload),
+            'wsgi.input': payload,
+        })
+        self.assertEqual(request.POST, {'name': ['Hello Günter']})
+        request.encoding = 'iso-8859-16'
+        self.assertEqual(request.POST, {'name': ['Hello GĂŒnter']})
+
+    def test_set_encoding_clears_GET(self):
+        request = WSGIRequest({
+            'REQUEST_METHOD': 'GET',
+            'wsgi.input': '',
+            'QUERY_STRING': b'name=Hello%20G%C3%BCnter' if six.PY2 else 'name=Hello%20G%C3%BCnter'
+        })
+        self.assertEqual(request.GET, {'name': ['Hello Günter']})
+        request.encoding = 'iso-8859-16'
+        self.assertEqual(request.GET, {'name': ['Hello G\u0102\u0152nter']})
+
     def test_FILES_connection_error(self):
         """
         If wsgi.input.read() raises an exception while trying to read() the
@@ -589,7 +637,7 @@ class HostValidationTests(SimpleTestCase):
         ALLOWED_HOSTS=[
             'forward.com', 'example.com', 'internal.com', '12.34.56.78',
             '[2001:19f0:feee::dead:beef:cafe]', 'xn--4ca9at.com',
-            '.multitenant.com', 'INSENSITIVE.com',
+            '.multitenant.com', 'INSENSITIVE.com', '[::ffff:169.254.169.254]',
         ])
     def test_http_get_host(self):
         # Check if X_FORWARDED_HOST is provided.
@@ -641,6 +689,7 @@ class HostValidationTests(SimpleTestCase):
             'insensitive.com',
             'example.com.',
             'example.com.:80',
+            '[::ffff:169.254.169.254]',
         ]
 
         for host in legit_hosts:
@@ -813,6 +862,11 @@ class HostValidationTests(SimpleTestCase):
         with self.assertRaisesMessage(SuspiciousOperation, msg_suggestion2 % "invalid_hostname.com"):
             request.get_host()
 
+    def test_split_domain_port_removes_trailing_dot(self):
+        domain, port = split_domain_port('example.com.:8080')
+        self.assertEqual(domain, 'example.com')
+        self.assertEqual(port, '8080')
+
 
 class BuildAbsoluteURITestCase(SimpleTestCase):
     """
@@ -824,8 +878,8 @@ class BuildAbsoluteURITestCase(SimpleTestCase):
 
     def test_build_absolute_uri_no_location(self):
         """
-        Ensures that ``request.build_absolute_uri()`` returns the proper value
-        when the ``location`` argument is not provided, and ``request.path``
+        ``request.build_absolute_uri()`` returns the proper value when
+        the ``location`` argument is not provided, and ``request.path``
         begins with //.
         """
         # //// is needed to create a request with a path beginning with //
@@ -837,9 +891,9 @@ class BuildAbsoluteURITestCase(SimpleTestCase):
 
     def test_build_absolute_uri_absolute_location(self):
         """
-        Ensures that ``request.build_absolute_uri()`` returns the proper value
-        when an absolute URL ``location`` argument is provided, and
-        ``request.path`` begins with //.
+        ``request.build_absolute_uri()`` returns the proper value when
+        an absolute URL ``location`` argument is provided, and ``request.path``
+        begins with //.
         """
         # //// is needed to create a request with a path beginning with //
         request = self.factory.get('////absolute-uri')
@@ -850,8 +904,8 @@ class BuildAbsoluteURITestCase(SimpleTestCase):
 
     def test_build_absolute_uri_schema_relative_location(self):
         """
-        Ensures that ``request.build_absolute_uri()`` returns the proper value
-        when a schema-relative URL ``location`` argument is provided, and
+        ``request.build_absolute_uri()`` returns the proper value when
+        a schema-relative URL ``location`` argument is provided, and
         ``request.path`` begins with //.
         """
         # //// is needed to create a request with a path beginning with //
@@ -863,9 +917,9 @@ class BuildAbsoluteURITestCase(SimpleTestCase):
 
     def test_build_absolute_uri_relative_location(self):
         """
-        Ensures that ``request.build_absolute_uri()`` returns the proper value
-        when a relative URL ``location`` argument is provided, and
-        ``request.path`` begins with //.
+        ``request.build_absolute_uri()`` returns the proper value when
+        a relative URL ``location`` argument is provided, and ``request.path``
+        begins with //.
         """
         # //// is needed to create a request with a path beginning with //
         request = self.factory.get('////absolute-uri')
