@@ -1,13 +1,10 @@
-from __future__ import absolute_import
-
-from django.conf.urls import patterns, url
+from django.conf.urls import url
 
 from . import views
 
-
-urlpatterns = patterns('',
+urlpatterns = [
     url(r'^example_view/$', views.example_view),
     url(r'^model_view/$', views.model_view),
     url(r'^create_model_instance/$', views.create_model_instance),
     url(r'^environ_view/$', views.environ_view),
-)
+]
