@@ -7,8 +7,7 @@ from ..utils import setup
 
 class CenterTests(SimpleTestCase):
 
-    @setup({'center01':
-        '{% autoescape off %}.{{ a|center:"5" }}. .{{ b|center:"5" }}.{% endautoescape %}'})
+    @setup({'center01': '{% autoescape off %}.{{ a|center:"5" }}. .{{ b|center:"5" }}.{% endautoescape %}'})
     def test_center01(self):
         output = self.engine.render_to_string('center01', {"a": "a&b", "b": mark_safe("a&b")})
         self.assertEqual(output, ". a&b . . a&b .")
