@@ -1,5 +1,6 @@
 from django.contrib.sessions.backends.base import SessionBase
 
+
 class SessionStore(SessionBase):
     """
     A simple cookie-based session storage implementation.
@@ -25,6 +26,6 @@ class SessionStore(SessionBase):
     def load(self):
         try:
             return self.decode(self.session_key)
-        except:
+        except Exception:
             self.modified = True
             return {}
