@@ -8,6 +8,8 @@ from contextlib import contextmanager
 from unittest import SkipTest, skipIf
 from xml.dom.minidom import parseString
 
+import pytz
+
 from django.contrib.auth.models import User
 from django.core import serializers
 from django.core.exceptions import ImproperlyConfigured
@@ -34,13 +36,6 @@ from .models import (
     AllDayEvent, Event, MaybeEvent, Session, SessionEvent, Timestamp,
 )
 
-try:
-    import pytz
-except ImportError:
-    pytz = None
-
-requires_pytz = skipIf(pytz is None, "this test requires pytz")
-
 # These tests use the EAT (Eastern Africa Time) and ICT (Indochina Time)
 # who don't have Daylight Saving Time, so we can represent them easily
 # with FixedOffset, and use them directly as tzinfo in the constructors.
@@ -201,42 +196,30 @@ class LegacyDatabaseTests(TestCase):
     def test_query_datetimes(self):
         Event.objects.create(dt=datetime.datetime(2011, 1, 1, 1, 30, 0))
         Event.objects.create(dt=datetime.datetime(2011, 1, 1, 4, 30, 0))
-        self.assertQuerysetEqual(
-            Event.objects.datetimes('dt', 'year'),
-            [datetime.datetime(2011, 1, 1, 0, 0, 0)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
-            Event.objects.datetimes('dt', 'month'),
-            [datetime.datetime(2011, 1, 1, 0, 0, 0)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
-            Event.objects.datetimes('dt', 'day'),
-            [datetime.datetime(2011, 1, 1, 0, 0, 0)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(Event.objects.datetimes('dt', 'year'), [datetime.datetime(2011, 1, 1, 0, 0, 0)])
+        self.assertSequenceEqual(Event.objects.datetimes('dt', 'month'), [datetime.datetime(2011, 1, 1, 0, 0, 0)])
+        self.assertSequenceEqual(Event.objects.datetimes('dt', 'day'), [datetime.datetime(2011, 1, 1, 0, 0, 0)])
+        self.assertSequenceEqual(
             Event.objects.datetimes('dt', 'hour'),
             [datetime.datetime(2011, 1, 1, 1, 0, 0),
-             datetime.datetime(2011, 1, 1, 4, 0, 0)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
+             datetime.datetime(2011, 1, 1, 4, 0, 0)]
+        )
+        self.assertSequenceEqual(
             Event.objects.datetimes('dt', 'minute'),
             [datetime.datetime(2011, 1, 1, 1, 30, 0),
-             datetime.datetime(2011, 1, 1, 4, 30, 0)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
+             datetime.datetime(2011, 1, 1, 4, 30, 0)]
+        )
+        self.assertSequenceEqual(
             Event.objects.datetimes('dt', 'second'),
             [datetime.datetime(2011, 1, 1, 1, 30, 0),
-             datetime.datetime(2011, 1, 1, 4, 30, 0)],
-            transform=lambda d: d)
+             datetime.datetime(2011, 1, 1, 4, 30, 0)]
+        )
 
     def test_raw_sql(self):
         # Regression test for #17755
         dt = datetime.datetime(2011, 9, 1, 13, 20, 30)
         event = Event.objects.create(dt=dt)
-        self.assertQuerysetEqual(
-            Event.objects.raw('SELECT * FROM timezones_event WHERE dt = %s', [dt]),
-            [event],
-            transform=lambda d: d)
+        self.assertSequenceEqual(list(Event.objects.raw('SELECT * FROM timezones_event WHERE dt = %s', [dt])), [event])
 
     def test_cursor_execute_accepts_naive_datetime(self):
         dt = datetime.datetime(2011, 9, 1, 13, 20, 30)
@@ -376,7 +359,6 @@ class NewDatabaseTests(TestCase):
         self.assertEqual(Event.objects.filter(dt__gte=dt2).count(), 1)
         self.assertEqual(Event.objects.filter(dt__gt=dt2).count(), 0)
 
-    @requires_pytz
     def test_query_filter_with_pytz_timezones(self):
         tz = pytz.timezone('Europe/Paris')
         dt = datetime.datetime(2011, 9, 1, 12, 20, 30, tzinfo=tz)
@@ -470,78 +452,75 @@ class NewDatabaseTests(TestCase):
     def test_query_datetimes(self):
         Event.objects.create(dt=datetime.datetime(2011, 1, 1, 1, 30, 0, tzinfo=EAT))
         Event.objects.create(dt=datetime.datetime(2011, 1, 1, 4, 30, 0, tzinfo=EAT))
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Event.objects.datetimes('dt', 'year'),
-            [datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=EAT)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
+            [datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=EAT)]
+        )
+        self.assertSequenceEqual(
             Event.objects.datetimes('dt', 'month'),
-            [datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=EAT)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
+            [datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=EAT)]
+        )
+        self.assertSequenceEqual(
             Event.objects.datetimes('dt', 'day'),
-            [datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=EAT)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
+            [datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=EAT)]
+        )
+        self.assertSequenceEqual(
             Event.objects.datetimes('dt', 'hour'),
             [datetime.datetime(2011, 1, 1, 1, 0, 0, tzinfo=EAT),
-             datetime.datetime(2011, 1, 1, 4, 0, 0, tzinfo=EAT)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
+             datetime.datetime(2011, 1, 1, 4, 0, 0, tzinfo=EAT)]
+        )
+        self.assertSequenceEqual(
             Event.objects.datetimes('dt', 'minute'),
             [datetime.datetime(2011, 1, 1, 1, 30, 0, tzinfo=EAT),
-             datetime.datetime(2011, 1, 1, 4, 30, 0, tzinfo=EAT)],
-            transform=lambda d: d)
-        self.assertQuerysetEqual(
+             datetime.datetime(2011, 1, 1, 4, 30, 0, tzinfo=EAT)]
+        )
+        self.assertSequenceEqual(
             Event.objects.datetimes('dt', 'second'),
             [datetime.datetime(2011, 1, 1, 1, 30, 0, tzinfo=EAT),
-             datetime.datetime(2011, 1, 1, 4, 30, 0, tzinfo=EAT)],
-            transform=lambda d: d)
+             datetime.datetime(2011, 1, 1, 4, 30, 0, tzinfo=EAT)]
+        )
 
     @skipUnlessDBFeature('has_zoneinfo_database')
     def test_query_datetimes_in_other_timezone(self):
         Event.objects.create(dt=datetime.datetime(2011, 1, 1, 1, 30, 0, tzinfo=EAT))
         Event.objects.create(dt=datetime.datetime(2011, 1, 1, 4, 30, 0, tzinfo=EAT))
         with timezone.override(UTC):
-            self.assertQuerysetEqual(
+            self.assertSequenceEqual(
                 Event.objects.datetimes('dt', 'year'),
                 [datetime.datetime(2010, 1, 1, 0, 0, 0, tzinfo=UTC),
-                 datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=UTC)],
-                transform=lambda d: d)
-            self.assertQuerysetEqual(
+                 datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=UTC)]
+            )
+            self.assertSequenceEqual(
                 Event.objects.datetimes('dt', 'month'),
                 [datetime.datetime(2010, 12, 1, 0, 0, 0, tzinfo=UTC),
-                 datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=UTC)],
-                transform=lambda d: d)
-            self.assertQuerysetEqual(
+                 datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=UTC)]
+            )
+            self.assertSequenceEqual(
                 Event.objects.datetimes('dt', 'day'),
                 [datetime.datetime(2010, 12, 31, 0, 0, 0, tzinfo=UTC),
-                 datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=UTC)],
-                transform=lambda d: d)
-            self.assertQuerysetEqual(
+                 datetime.datetime(2011, 1, 1, 0, 0, 0, tzinfo=UTC)]
+            )
+            self.assertSequenceEqual(
                 Event.objects.datetimes('dt', 'hour'),
                 [datetime.datetime(2010, 12, 31, 22, 0, 0, tzinfo=UTC),
-                 datetime.datetime(2011, 1, 1, 1, 0, 0, tzinfo=UTC)],
-                transform=lambda d: d)
-            self.assertQuerysetEqual(
+                 datetime.datetime(2011, 1, 1, 1, 0, 0, tzinfo=UTC)]
+            )
+            self.assertSequenceEqual(
                 Event.objects.datetimes('dt', 'minute'),
                 [datetime.datetime(2010, 12, 31, 22, 30, 0, tzinfo=UTC),
-                 datetime.datetime(2011, 1, 1, 1, 30, 0, tzinfo=UTC)],
-                transform=lambda d: d)
-            self.assertQuerysetEqual(
+                 datetime.datetime(2011, 1, 1, 1, 30, 0, tzinfo=UTC)]
+            )
+            self.assertSequenceEqual(
                 Event.objects.datetimes('dt', 'second'),
                 [datetime.datetime(2010, 12, 31, 22, 30, 0, tzinfo=UTC),
-                 datetime.datetime(2011, 1, 1, 1, 30, 0, tzinfo=UTC)],
-                transform=lambda d: d)
+                 datetime.datetime(2011, 1, 1, 1, 30, 0, tzinfo=UTC)]
+            )
 
     def test_raw_sql(self):
         # Regression test for #17755
         dt = datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=EAT)
         event = Event.objects.create(dt=dt)
-        self.assertQuerysetEqual(
-            Event.objects.raw('SELECT * FROM timezones_event WHERE dt = %s', [dt]),
-            [event],
-            transform=lambda d: d)
+        self.assertSequenceEqual(list(Event.objects.raw('SELECT * FROM timezones_event WHERE dt = %s', [dt])), [event])
 
     @skipUnlessDBFeature('supports_timezones')
     def test_cursor_execute_accepts_aware_datetime(self):
@@ -700,7 +679,7 @@ class SerializationTests(SimpleTestCase):
 
     def assert_yaml_contains_datetime(self, yaml, dt):
         # Depending on the yaml dumper, '!timestamp' might be absent
-        six.assertRegex(self, yaml, r"\n  fields: {dt: !(!timestamp)? '%s'}" % re.escape(dt))
+        self.assertRegex(yaml, r"\n  fields: {dt: !(!timestamp)? '%s'}" % re.escape(dt))
 
     def test_naive_datetime(self):
         dt = datetime.datetime(2011, 9, 1, 13, 20, 30)
@@ -848,7 +827,7 @@ class SerializationTests(SimpleTestCase):
 
 
 @override_settings(DATETIME_FORMAT='c', TIME_ZONE='Africa/Nairobi', USE_L10N=False, USE_TZ=True)
-class TemplateTests(TestCase):
+class TemplateTests(SimpleTestCase):
 
     @requires_tz_support
     def test_localtime_templatetag_and_filters(self):
@@ -931,7 +910,6 @@ class TemplateTests(TestCase):
                     expected = results[k1][k2]
                     self.assertEqual(actual, expected, '%s / %s: %r != %r' % (k1, k2, actual, expected))
 
-    @requires_pytz
     def test_localtime_filters_with_pytz(self):
         """
         Test the |localtime, |utc, and |timezone filters with pytz.
@@ -999,7 +977,6 @@ class TemplateTests(TestCase):
             "2011-09-01T13:20:30+03:00|2011-09-01T17:20:30+07:00|2011-09-01T13:20:30+03:00"
         )
 
-    @requires_pytz
     def test_timezone_templatetag_with_pytz(self):
         """
         Test the {% timezone %} templatetag with pytz.
@@ -1019,7 +996,7 @@ class TemplateTests(TestCase):
     def test_timezone_templatetag_invalid_argument(self):
         with self.assertRaises(TemplateSyntaxError):
             Template("{% load tz %}{% timezone %}{% endtimezone %}").render()
-        with self.assertRaises(ValueError if pytz is None else pytz.UnknownTimeZoneError):
+        with self.assertRaises(pytz.UnknownTimeZoneError):
             Template("{% load tz %}{% timezone tz %}{% endtimezone %}").render(Context({'tz': 'foobar'}))
 
     @skipIf(sys.platform.startswith('win'), "Windows uses non-standard time zone names")
@@ -1029,7 +1006,7 @@ class TemplateTests(TestCase):
         """
         tpl = Template("{% load tz %}{% get_current_timezone as time_zone %}{{ time_zone }}")
 
-        self.assertEqual(tpl.render(Context()), "Africa/Nairobi" if pytz else "EAT")
+        self.assertEqual(tpl.render(Context()), "Africa/Nairobi")
         with timezone.override(UTC):
             self.assertEqual(tpl.render(Context()), "UTC")
 
@@ -1042,7 +1019,6 @@ class TemplateTests(TestCase):
         with timezone.override(UTC):
             self.assertEqual(tpl.render(Context({'tz': ICT})), "+0700")
 
-    @requires_pytz
     def test_get_current_timezone_templatetag_with_pytz(self):
         """
         Test the {% get_current_timezone %} templatetag with pytz.
@@ -1071,7 +1047,7 @@ class TemplateTests(TestCase):
         context = Context()
         self.assertEqual(tpl.render(context), "")
         request_context = RequestContext(HttpRequest(), processors=[context_processors.tz])
-        self.assertEqual(tpl.render(request_context), "Africa/Nairobi" if pytz else "EAT")
+        self.assertEqual(tpl.render(request_context), "Africa/Nairobi")
 
     @requires_tz_support
     def test_date_and_time_template_filters(self):
@@ -1091,15 +1067,6 @@ class TemplateTests(TestCase):
         with timezone.override(ICT):
             self.assertEqual(tpl.render(ctx), "2011-09-01 at 20:20:20")
 
-    def test_localtime_with_time_zone_setting_set_to_none(self):
-        # Regression for #17274
-        tpl = Template("{% load tz %}{{ dt }}")
-        ctx = Context({'dt': datetime.datetime(2011, 9, 1, 12, 20, 30, tzinfo=EAT)})
-
-        with self.settings(TIME_ZONE=None):
-            # the actual value depends on the system time zone of the host
-            self.assertTrue(tpl.render(ctx).startswith("2011"))
-
     @requires_tz_support
     def test_now_template_tag_uses_current_time_zone(self):
         # Regression for #17343
@@ -1117,7 +1084,6 @@ class LegacyFormsTests(TestCase):
         self.assertTrue(form.is_valid())
         self.assertEqual(form.cleaned_data['dt'], datetime.datetime(2011, 9, 1, 13, 20, 30))
 
-    @requires_pytz
     def test_form_with_non_existent_time(self):
         form = EventForm({'dt': '2011-03-27 02:30:00'})
         with timezone.override(pytz.timezone('Europe/Paris')):
@@ -1125,7 +1091,6 @@ class LegacyFormsTests(TestCase):
             self.assertTrue(form.is_valid())
             self.assertEqual(form.cleaned_data['dt'], datetime.datetime(2011, 3, 27, 2, 30, 0))
 
-    @requires_pytz
     def test_form_with_ambiguous_time(self):
         form = EventForm({'dt': '2011-10-30 02:30:00'})
         with timezone.override(pytz.timezone('Europe/Paris')):
@@ -1164,7 +1129,6 @@ class NewFormsTests(TestCase):
         # Datetime inputs formats don't allow providing a time zone.
         self.assertFalse(form.is_valid())
 
-    @requires_pytz
     def test_form_with_non_existent_time(self):
         with timezone.override(pytz.timezone('Europe/Paris')):
             form = EventForm({'dt': '2011-03-27 02:30:00'})
@@ -1176,7 +1140,6 @@ class NewFormsTests(TestCase):
                 ]
             )
 
-    @requires_pytz
     def test_form_with_ambiguous_time(self):
         with timezone.override(pytz.timezone('Europe/Paris')):
             form = EventForm({'dt': '2011-10-30 02:30:00'})
