@@ -15,8 +15,8 @@ class TestUtilsHtml(SimpleTestCase):
 
     def check_output(self, function, value, output=None):
         """
-        Check that function(value) equals output.  If output is None,
-        check that function(value) equals value.
+        function(value) equals output. If output is None, function(value)
+        equals value.
         """
         if output is None:
             output = value
@@ -142,6 +142,7 @@ class TestUtilsHtml(SimpleTestCase):
                 'paragraph separator:\u2029and line separator:\u2028',
                 'paragraph separator:\\u2029and line separator:\\u2028'
             ),
+            ('`', '\\u0060'),
         )
         for value, output in items:
             self.check_output(f, value, output)
@@ -149,10 +150,10 @@ class TestUtilsHtml(SimpleTestCase):
 
     def test_smart_urlquote(self):
         quote = html.smart_urlquote
-        # Ensure that IDNs are properly quoted
+        # IDNs are properly quoted
         self.assertEqual(quote('http://öäü.com/'), 'http://xn--4ca9at.com/')
         self.assertEqual(quote('http://öäü.com/öäü/'), 'http://xn--4ca9at.com/%C3%B6%C3%A4%C3%BC/')
-        # Ensure that everything unsafe is quoted, !*'();:@&=+$,/?#[]~ is considered safe as per RFC
+        # Everything unsafe is quoted, !*'();:@&=+$,/?#[]~ is considered safe as per RFC
         self.assertEqual(quote('http://example.com/path/öäü/'), 'http://example.com/path/%C3%B6%C3%A4%C3%BC/')
         self.assertEqual(quote('http://example.com/%C3%B6/ä/'), 'http://example.com/%C3%B6/%C3%A4/')
         self.assertEqual(quote('http://example.com/?x=1&y=2+3&z='), 'http://example.com/?x=1&y=2+3&z=')
@@ -231,3 +232,11 @@ class TestUtilsHtml(SimpleTestCase):
             @html.html_safe
             class HtmlClass(object):
                 pass
+
+    def test_urlize_unchanged_inputs(self):
+        tests = (
+            ('a' + '@a' * 50000) + 'a',  # simple_email_re catastrophic test
+            ('a' + '.' * 1000000) + 'a',  # trailing_punctuation catastrophic test
+        )
+        for value in tests:
+            self.assertEqual(html.urlize(value), value)
