@@ -54,6 +54,23 @@ class I18NTests(TestCase):
         self.assertEqual(response.url, '/')
         self.assertEqual(self.client.session[LANGUAGE_SESSION_KEY], lang_code)
 
+    def test_setlang_http_next(self):
+        """
+        The set_language view only redirects to the 'next' argument if it is
+        "safe" and its scheme is https if the request was sent over https.
+        """
+        lang_code = self._get_inactive_language_code()
+        non_https_next_url = 'http://testserver/redirection/'
+        post_data = dict(language=lang_code, next=non_https_next_url)
+        # Insecure URL in POST data.
+        response = self.client.post('/i18n/setlang/', data=post_data, secure=True)
+        self.assertEqual(response.url, '/')
+        self.assertEqual(self.client.session[LANGUAGE_SESSION_KEY], lang_code)
+        # Insecure URL in HTTP referer.
+        response = self.client.post('/i18n/setlang/', secure=True, HTTP_REFERER=non_https_next_url)
+        self.assertEqual(response.url, '/')
+        self.assertEqual(self.client.session[LANGUAGE_SESSION_KEY], lang_code)
+
     def test_setlang_redirect_to_referer(self):
         """
         The set_language view redirects to the URL in the referer header when
@@ -124,13 +141,13 @@ class I18NTests(TestCase):
     def test_setlang_cookie(self):
         # we force saving language to a cookie rather than a session
         # by excluding session middleware and those which do require it
-        test_settings = dict(
-            MIDDLEWARE=['django.middleware.common.CommonMiddleware'],
-            LANGUAGE_COOKIE_NAME='mylanguage',
-            LANGUAGE_COOKIE_AGE=3600 * 7 * 2,
-            LANGUAGE_COOKIE_DOMAIN='.example.com',
-            LANGUAGE_COOKIE_PATH='/test/',
-        )
+        test_settings = {
+            'MIDDLEWARE': ['django.middleware.common.CommonMiddleware'],
+            'LANGUAGE_COOKIE_NAME': 'mylanguage',
+            'LANGUAGE_COOKIE_AGE': 3600 * 7 * 2,
+            'LANGUAGE_COOKIE_DOMAIN': '.example.com',
+            'LANGUAGE_COOKIE_PATH': '/test/',
+        }
         with self.settings(**test_settings):
             post_data = dict(language='pl', next='/views/')
             response = self.client.post('/i18n/setlang/', data=post_data)
