@@ -20,8 +20,7 @@ class WSGITest(SimpleTestCase):
 
     def test_get_wsgi_application(self):
         """
-        Verify that ``get_wsgi_application`` returns a functioning WSGI
-        callable.
+        get_wsgi_application() returns a functioning WSGI callable.
         """
         application = get_wsgi_application()
 
@@ -41,15 +40,16 @@ class WSGITest(SimpleTestCase):
 
         self.assertEqual(response_data["status"], "200 OK")
         self.assertEqual(
-            response_data["headers"],
-            [('Content-Type', 'text/html; charset=utf-8')])
-        self.assertEqual(
-            bytes(response),
-            b"Content-Type: text/html; charset=utf-8\r\n\r\nHello World!")
+            set(response_data["headers"]),
+            {('Content-Length', '12'), ('Content-Type', 'text/html; charset=utf-8')})
+        self.assertIn(bytes(response), [
+            b"Content-Length: 12\r\nContent-Type: text/html; charset=utf-8\r\n\r\nHello World!",
+            b"Content-Type: text/html; charset=utf-8\r\nContent-Length: 12\r\n\r\nHello World!"
+        ])
 
     def test_file_wrapper(self):
         """
-        Verify that FileResponse uses wsgi.file_wrapper.
+        FileResponse uses wsgi.file_wrapper.
         """
         class FileWrapper(object):
             def __init__(self, filelike, blksize=8192):
